/******************************************************************************
 *  File: calibration.c
 *  Description:
 *  Author: Arnaud Hugron
 *  Date: 08/13/2015
 *****************************************************************************/
#include <stdint.h>
#include "main.h"
#include "pmbus.h"
#include "analog.h"
#include "calibration.h"
#include "control.h"
#include "flashInterface.h"
#include "power.h"
#include "state.h"
#include "restart.h"
#include "preload.h"
#include "crc8.h"

#ifdef O3_CODE
#pragma GCC optimize ("O3")
#endif

#define CALIBRATION_VERSION                     1u
#define VALID_DATA                              0xFFFFFFFFu
#define INVALID_DATA                            0x00000000u

static /*__attribute__ ((section(".cal_dynamic_user1"))) TBD */ calibration_dynamic_t calibration_dynamic1;

// RAM copy of flash calibration
calibration_static_t calibration_static_ram;
calibration_dynamic_t calibration_dynamic_ram;

cal_data_t cal_vout_adj_10_percent_data;
cal_data_t cal_vout_adj_105_percent_data;

// in code space
const calibration_static_t calibration_static_default =
{
	{ VALID_DATA, 0u, 0u, CAL_STATIC_SIGNATURE },
	{MFR_ID_DFLT},
	{MFR_MODEL_DFLT},
	{MFR_REVISION},
	{MFR_LOCATION_DFLT},
	{MFR_DATE_DFLT},
	{MFR_SERIAL_DFLT},

	VOLTAGE_Kp,
	VOLTAGE_Ki,
	VOLTAGE_Kd,
	CURRENT_Kp,
	CURRENT_Ki,
	CURRENT_Kd,
	ISHARE_Kp,
	ISHARE_Ki,
	ISHARE_Kd,

	VOLTAGE_KAlpha,
	VOLTAGE_KBeta_UP,
	VOLTAGE_KBeta_DOWN,
	VOLTAGE_KBeta_OFS,

	VOLTAGE_COT_Kp,
	VOLTAGE_COT_Ki,
	VOLTAGE_COT_Kd,

#ifdef HPT5K_48V
	{{80,0}, {1065,614}, {32760,15114}},							// vout (VO_SENSE_H) calibration {adc, RVU}
	{{10219,-817570}, {7495,2083773}},

	{{300,0}, {1531,166}, {6123,666}, {12859,1666}, {25718,3333}},	// iout calibration {adc, RVI}
	{{2218,-665475}, {1784,-594}, {2432,-3970331}, {2123,0}},

	{{400,0}, {6146,222}, {24585,1111}},							// iout1 calibration {adc, RVI}
	{{633,-253455}, {790,-1213366}},
	{{400,0}, {6146,222}, {24585,1111}},							// iout2 calibration {adc, RVI}
	{{633,-253455}, {790,-1213366}},
	{{400,0}, {6146,222}, {24585,1111}},							// iout3 calibration {adc, RVI}
	{{633,-253455}, {790,-1213366}},

#elif HPT5K_60V
	{{60,0}, {1065,614}, {32760,18893}},							// vout (VO_SENSE_H) calibration {adc, RVU}
	{{10010,-600600}, {9449,272891}},

	{{300,0}, {1531,160}, {6123,640}, {12859,1344}, {25718,2688}},	// iout calibration
	{{2130,-639000}, {1713,70837}, {1712,291184}, {1712,610288}},

	{{1000,32}, {4593,160}, {25723,896}},							// iout1 calibration
	{{584,-45312}, {571,70837}},
	{{1000,32}, {4593,160}, {25723,896}},							// iout2 calibration
	{{584,-45312}, {571,70837}},
	{{1000,32}, {4593,160}, {25723,896}},							// iout3 calibration
	{{584,-45312}, {571,70837}},

#elif HPT5K_80V
	{{80,0}, {1331,1024}, {32760,25190}},							// vout (VO_SENSE_H) calibration {adc, RVU}
	{{13403,-1072277}, {12598, 0}},

	{{300,0}, {1531,100}, {6123,400}, {12859,1000}, {25718,2000}},	// iout calibration
	{{1331,-399285}, {1070,-357}, {1459,-2382198}, {1274,0}},

	{{400,0}, {6146,133}, {24585,666}},								// iout1 calibration
	{{380,-152073}, {474,-728019}},
	{{400,0}, {6146,133}, {24585,666}},								// iout2 calibration
	{{380,-152073}, {474,-728019}},
	{{400,0}, {6146,133}, {24585,666}},								// iout3 calibration
	{{380,-152073}, {474,-728019}},

#elif HPT5K_100V
	{{60,0}, {1059,1024}, {32760,31675}},							// vout (VO_SENSE_H) calibration {adc, RVU}
	{{16794,-1007640}, {15841, 462397}},

	{{300,0}, {2559,160}, {5119,320}, {12797,800}, {25594,1600}},	// iout calibration
	{{1160,-348000}, {1024,73024}, {1024,145024}, {1024,363072}},

	{{1536,32}, {6146,128}, {24585,512}},							// iout1 calibration
	{{341,14912}, {341,58966}},
	{{1536,32}, {6146,128}, {24585,512}},							// iout2 calibration
	{{341,14912}, {341,58966}},
	{{1536,32}, {6146,128}, {24585,512}},							// iout3 calibration
	{{341,14912}, {341,58966}},

#elif HPT5K_200V
	{{60,0}, {1065,2048}, {32760,62976}},							// vout (VO_SENSE_H) calibration {adc, RVU}
	{{33387,-2003220}, {31495,933857}},

	{{300,0}, {2559,80}, {5119,160}, {12285,384}, {25594,800}},		// iout calibration
	{{580,-174000}, {512,36512}, {512,72512}, {512,174336}},

	{{1536,16}, {6146,64}, {24585,256}},							// iout1 calibration
	{{171,6688}, {171,26410}},
	{{1536,16}, {6146,64}, {24585,256}},							// iout2 calibration
	{{171,6688}, {171,26410}},
	{{1536,16}, {6146,64}, {24585,256}},							// iout3 calibration
	{{171,6688}, {171,26410}},

#elif HPT5K_400V
    {{60,0}, {1331,2560}, {26634,51200}},                           // vout (VO_SENSE_H) calibration {adc, RVU}
    {{32982,-1978920}, {31496,1151586}},

    {{200,0}, {1280,160}, {5119,640}, {12797,1600}, {25594,3200}},     // iout calibration {adc, RVI}
    {{2428,-485600}, {2049,71360}, {2049,285441}, {2049,713603}},

    {{1272,53}, {6361,265}, {25446,1060}},                            // iout1 calibration {adc, RVI}
    {{682,24363}, {682,121818}},
    {{1272,53}, {6361,265}, {25446,1060}},                            // iout2 calibration {adc, RVI}
    {{682,24363}, {682,121818}},
    {{1272,53}, {6361,265}, {25446,1060}},                            // iout3 calibration {adc, RVI}
    {{682,24363}, {682,121818}},

#elif HPT5K_800V
    {{60,0}, {1331,2560}, {26634,51200}},                           // vout (VO_SENSE_H) calibration {adc, RVU}
    {{32982,-1978920}, {31496,1151586}},

    {{200,0}, {1280,160}, {5119,640}, {12797,1600}, {25594,3200}},     // iout calibration {adc, RVI}
    {{2428,-485600}, {2049,71360}, {2049,285441}, {2049,713603}},

    {{1272,53}, {6361,265}, {25446,1060}},                            // iout1 calibration {adc, RVI}
    {{682,24363}, {682,121818}},
    {{1272,53}, {6361,265}, {25446,1060}},                            // iout2 calibration {adc, RVI}
    {{682,24363}, {682,121818}},
    {{1272,53}, {6361,265}, {25446,1060}},                            // iout3 calibration {adc, RVI}
    {{682,24363}, {682,121818}},
#endif
	0,						// vprog_offset
	0,						// iprog_offset
	ISHARE_OFS_DFLT,		// ishare_offset
	0,						// pfc1_offset
	0,						// pfc2_offset
	0,						// pfc3_offset

	{{150,65}, {15438,13968}, {30876,27936}},						// vprog calibration
	{{14821,93787}, {14821,93787}},
	{{100,120}, {15438,14280}, {30876,27920}},						// iprog calibration
	{{15125,453513}, {14475,10485760}},
};

//***************************************************************************************************
// in data flash
/*__attribute__ ((section(".cal_static")))*/ calibration_static_t calibration_static =
{
	{ VALID_DATA, 0u, 0u, CAL_STATIC_SIGNATURE },
	{MFR_ID_DFLT},
	{MFR_MODEL_DFLT},
	{MFR_REVISION},
	{MFR_LOCATION_DFLT},
	{MFR_DATE_DFLT},
	{MFR_SERIAL_DFLT},

	VOLTAGE_Kp,
	VOLTAGE_Ki,
	VOLTAGE_Kd,
	CURRENT_Kp,
	CURRENT_Ki,
	CURRENT_Kd,
	ISHARE_Kp,
	ISHARE_Ki,
	ISHARE_Kd,

	VOLTAGE_KAlpha,
	VOLTAGE_KBeta_UP,
	VOLTAGE_KBeta_DOWN,
	VOLTAGE_KBeta_OFS,

	VOLTAGE_COT_Kp,
	VOLTAGE_COT_Ki,
	VOLTAGE_COT_Kd,

#ifdef HPT5K_48V
	{{80,0}, {1065,614}, {32760,15114}},							// vout (VO_SENSE_H) calibration {adc, RVU}
	{{10219,-817570}, {7495,2083773}},

	{{300,0}, {1531,166}, {6123,666}, {12859,1666}, {25718,3333}},	// iout calibration
	{{2218,-665475}, {1784,-594}, {2432,-3970331}, {2123,0}},

	{{400,0}, {6146,222}, {24585,1111}},							// iout1 calibration
	{{633,-253455}, {790,-1213366}},
	{{400,0}, {6146,222}, {24585,1111}},							// iout2 calibration
	{{633,-253455}, {790,-1213366}},
	{{400,0}, {6146,222}, {24585,1111}},							// iout3 calibration
	{{633,-253455}, {790,-1213366}},

#elif HPT5K_60V
	{{60,0}, {1065,614}, {32760,18893}},							// vout (VO_SENSE_H) calibration
	{{10010,-600600}, {9449,272891}},

	{{300,0}, {1531,160}, {6123,640}, {12859,1344}, {25718,2688}},	// iout calibration
	{{2130,-639000}, {1713,70837}, {1712,291184}, {1712,610288}},

	{{1000,32}, {4593,160}, {25723,896}},							// iout1 calibration
	{{584,-45312}, {571,70837}},
	{{1000,32}, {4593,160}, {25723,896}},							// iout2 calibration
	{{584,-45312}, {571,70837}},
	{{1000,32}, {4593,160}, {25723,896}},							// iout3 calibration
	{{584,-45312}, {571,70837}},

#elif HPT5K_80V
	{{80,0}, {1331,1024}, {32760,25190}},							// vout (VO_SENSE_H) calibration {adc, RVU}
	{{13403,-1072277}, {12598, 0}},

	{{300,0}, {1531,100}, {6123,400}, {12859,1000}, {25718,2000}},	// iout calibration
	{{1331,-399285}, {1070,-357}, {1459,-2382198}, {1274,0}},

	{{400,0}, {6146,133}, {24585,666}},								// iout1 calibration
	{{380,-152073}, {474,-728019}},
	{{400,0}, {6146,133}, {24585,666}},								// iout2 calibration
	{{380,-152073}, {474,-728019}},
	{{400,0}, {6146,133}, {24585,666}},								// iout3 calibration
	{{380,-152073}, {474,-728019}},

#elif HPT5K_100V
	{{60,0}, {1059,1024}, {32760,31675}},							// vout (VO_SENSE_H) calibration
	{{16794,-1007640}, {15841, 462397}},

	{{300,0}, {2559,160}, {5119,320}, {12797,800}, {25594,1600}},	// iout calibration
	{{1160,-348000}, {1024,73024}, {1024,145024}, {1024,363072}},

	{{1536,32}, {6146,128}, {24585,512}},							// iout1 calibration
	{{341,14912}, {341,58966}},
	{{1536,32}, {6146,128}, {24585,512}},							// iout2 calibration
	{{341,14912}, {341,58966}},
	{{1536,32}, {6146,128}, {24585,512}},							// iout3 calibration
	{{341,14912}, {341,58966}},

#elif HPT5K_200V
	{{60,0}, {1065,2048}, {32760,62976}},							// vout (VO_SENSE_H) calibration
	{{33387,-2003220}, {31495,933857}},

	{{300,0}, {2559,80}, {5119,160}, {12285,384}, {25594,800}},		// iout calibration
	{{580,-174000}, {512,36512}, {512,72512}, {512,174336}},

	{{1536,16}, {6146,64}, {24585,256}},							// iout1 calibration
	{{171,6688}, {171,26410}},
	{{1536,16}, {6146,64}, {24585,256}},							// iout2 calibration
	{{171,6688}, {171,26410}},
	{{1536,16}, {6146,64}, {24585,256}},							// iout3 calibration
	{{171,6688}, {171,26410}},

#elif HPT5K_400V
    {{60,0}, {1331,2560}, {26634,51200}},                           // vout (VO_SENSE_H) calibration {adc, RVU}
    {{32982,-1978920}, {31496,1151586}},

    {{200,0}, {1280,160}, {5119,640}, {12797,1600}, {25594,3200}},     // iout calibration {adc, RVI}
    {{2428,-485600}, {2049,71360}, {2049,285441}, {2049,713603}},

    {{1272,53}, {6361,265}, {25446,1060}},                            // iout1 calibration {adc, RVI}
    {{682,24363}, {682,121818}},
    {{1272,53}, {6361,265}, {25446,1060}},                            // iout2 calibration {adc, RVI}
    {{682,24363}, {682,121818}},
    {{1272,53}, {6361,265}, {25446,1060}},                            // iout3 calibration {adc, RVI}
    {{682,24363}, {682,121818}},

#elif HPT5K_800V
    {{60,0}, {1065,2048}, {32760,62976}},                           // vout (VO_SENSE_H) calibration {adc, RVU}
    {{33387,-2003220}, {31495,933857}},

    {{300,0}, {2559,80}, {5119,160}, {12285,384}, {25594,800}},     // iout calibration
    {{580,-174000}, {512,36512}, {512,72512}, {512,174336}},

    {{1536,16}, {6146,64}, {24585,256}},                            // iout1 calibration
    {{171,6688}, {171,26410}},
    {{1536,16}, {6146,64}, {24585,256}},                            // iout2 calibration
    {{171,6688}, {171,26410}},
    {{1536,16}, {6146,64}, {24585,256}},                            // iout3 calibration
    {{171,6688}, {171,26410}},
#endif
	0,						// vprog_offset
	0,						// iprog_offset
	ISHARE_OFS_DFLT,		// ishare_offset

	0,						// pfc1_offset
	0,						// pfc2_offset
	0,						// pfc3_offset

	{{150,65}, {15438,13968}, {30876,27936}},						// vprog calibration
	{{14821,93787}, {14821,93787}},
	{{100,120}, {15438,14280}, {30876,27920}},						// iprog calibration
	{{15125,453513}, {14475,10485760}},
};

// in code space
const calibration_dynamic_t calibration_dynamic_factory_default =
{
	{ VALID_DATA, 0u, 0u, CAL_DYNAMIC_SIGNATURE },
    {
		VSET_DFLT,

		VOUT_UV_FAULT_LIMIT_RVU,
		VOUT_UV_WARN_LIMIT_RVU,
		VOUT_OV_WARN_LIMIT_DEFAULT,
		VOUT_OV_FAULT_LIMIT_DEFAULT,			// linear format for all fault and warning

		IOUT_OC_WARN_LIMIT_DEFAULT,
		IOUT_OC_FAULT_LIMIT_DEFAULT,
		IOUT_OC_LV_FAULT_LIMIT_DEFAULT,
		IOUT_OC_FAULT_LIMIT_DEFAULT,			// pmbus iprog

		OT_SEC_WARN_LIMIT_DEFAULT,
		OT_SEC_FAULT_LIMIT_DEFAULT,
		OT_PRI_WARN_LIMIT_DEFAULT,
		OT_PRI_FAULT_LIMIT_DEFAULT,

		FAN_COMMAND_MIN,

		VIN_UV_FAULT_LIMIT_DEFAULT,
		VIN_UV_WARN_LIMIT_DEFAULT,
		VIN_OV_WARN_LIMIT_DEFAULT,
		VIN_OV_FAULT_LIMIT_DEFAULT,

		VOUT_UV_FAULT_RESPONSE_DEFAULT,
		IOUT_OC_FAULT_RESPONSE_DEFAULT,
		OT_FAULT_RESPONSE_DEFAULT,
		VIN_UV_FAULT_RESPONSE_DEFAULT,
		VIN_OV_FAULT_RESPONSE_DEFAULT,
		IOUTX_FAULT_RESPONSE_DEFAULT,

		OPERATION_DEFAULT,
		0x00,									// slave ID, 0x00, means use default 0xB0 with A2-A0, if different than 0x00, use this address ID
		PMBUS_BASE_SLAVE_ADDR,
		0x00,
        RAMP_VOLTAGE_MIN,                       // ramp up time
        RAMP_VOLTAGE_MIN,                       // ramp down time
        RAMP_CURRENT_MIN,                       // current ramp up time
        RAMP_CURRENT_MIN,                       // current ramp down time
        0,
        0
    },
	{USER_CONFIG_DEFAULT},
	{HARDWARE_CONFIG_DEFAULT},
	{0},
	{0}
};

// in data flash
/*__attribute__ ((section(".cal_dynamic_factory"))) */ calibration_dynamic_t calibration_dynamic_factory =
{
	{ VALID_DATA, 0u, 0u, CAL_DYNAMIC_SIGNATURE },
    {
		VSET_DFLT,

		VOUT_UV_FAULT_LIMIT_RVU,
		VOUT_UV_WARN_LIMIT_RVU,
		VOUT_OV_WARN_LIMIT_DEFAULT,
		VOUT_OV_FAULT_LIMIT_DEFAULT,			// linear format for all fault and warning

		IOUT_OC_WARN_LIMIT_DEFAULT,
		IOUT_OC_FAULT_LIMIT_DEFAULT,
		IOUT_OC_LV_FAULT_LIMIT_DEFAULT,
		IOUT_OC_FAULT_LIMIT_DEFAULT,			// pmbus iprog

		OT_SEC_WARN_LIMIT_DEFAULT,
		OT_SEC_FAULT_LIMIT_DEFAULT,
		OT_PRI_WARN_LIMIT_DEFAULT,
		OT_PRI_FAULT_LIMIT_DEFAULT,

		FAN_COMMAND_MIN,

		VIN_UV_FAULT_LIMIT_DEFAULT,
		VIN_UV_WARN_LIMIT_DEFAULT,
		VIN_OV_WARN_LIMIT_DEFAULT,
		VIN_OV_FAULT_LIMIT_DEFAULT,

		VOUT_UV_FAULT_RESPONSE_DEFAULT,
		IOUT_OC_FAULT_RESPONSE_DEFAULT,
		OT_FAULT_RESPONSE_DEFAULT,
		VIN_UV_FAULT_RESPONSE_DEFAULT,
		VIN_OV_FAULT_RESPONSE_DEFAULT,
		IOUTX_FAULT_RESPONSE_DEFAULT,

		OPERATION_DEFAULT,
		0x00,
		PMBUS_BASE_SLAVE_ADDR,
		0x00,
        RAMP_VOLTAGE_MIN,                       // ramp up time
        RAMP_VOLTAGE_MIN,                       // ramp down time
        RAMP_CURRENT_MIN,                       // current ramp up time
        RAMP_CURRENT_MIN,                       // current ramp down time
        0,
        0
    },
	{USER_CONFIG_DEFAULT},
	{HARDWARE_CONFIG_DEFAULT},
	{0},
    {0}
};

// in data flash
static /*__attribute__ ((section(".cal_dynamic_user0"))) */ calibration_dynamic_t calibration_dynamic0 =
{
	{ VALID_DATA, 0u, 0u, CAL_DYNAMIC_SIGNATURE },
    {
		VSET_DFLT,

		VOUT_UV_FAULT_LIMIT_RVU,
		VOUT_UV_WARN_LIMIT_RVU,
		VOUT_OV_WARN_LIMIT_DEFAULT,
		VOUT_OV_FAULT_LIMIT_DEFAULT,			// linear format for all fault and warning

		IOUT_OC_WARN_LIMIT_DEFAULT,
		IOUT_OC_FAULT_LIMIT_DEFAULT,
		IOUT_OC_LV_FAULT_LIMIT_DEFAULT,
		IOUT_OC_FAULT_LIMIT_DEFAULT,			// pmbus iprog

		OT_SEC_WARN_LIMIT_DEFAULT,
		OT_SEC_FAULT_LIMIT_DEFAULT,
		OT_PRI_WARN_LIMIT_DEFAULT,
		OT_PRI_FAULT_LIMIT_DEFAULT,

		FAN_COMMAND_MIN,

		VIN_UV_FAULT_LIMIT_DEFAULT,
		VIN_UV_WARN_LIMIT_DEFAULT,
		VIN_OV_WARN_LIMIT_DEFAULT,
		VIN_OV_FAULT_LIMIT_DEFAULT,

		VOUT_UV_FAULT_RESPONSE_DEFAULT,
		IOUT_OC_FAULT_RESPONSE_DEFAULT,
		OT_FAULT_RESPONSE_DEFAULT,
		VIN_UV_FAULT_RESPONSE_DEFAULT,
		VIN_OV_FAULT_RESPONSE_DEFAULT,
		IOUTX_FAULT_RESPONSE_DEFAULT,

		OPERATION_DEFAULT,
		0x00,
		PMBUS_BASE_SLAVE_ADDR,
		0x00,
        RAMP_VOLTAGE_MIN,                       // ramp up time
        RAMP_VOLTAGE_MIN,                       // ramp down time
        RAMP_CURRENT_MIN,                       // current ramp up time
        RAMP_CURRENT_MIN,                       // current ramp down time
        0,
        0
    },
	{USER_CONFIG_DEFAULT},
	{HARDWARE_CONFIG_DEFAULT},
	{0},
	{0}
};

void calibration_init(void)
{
	get_calibration_static();
	get_calibration_dynamic();
	calibration_init_dynamic_setting();
}

void calibration_init_dynamic_setting(void)
{
	calibration_vout_init();
	calibration_iout_init();
	calibration_temperature_init();
	calibration_input_init();
	calibration_ramp_init();
}

static calibration_static_t *calibration_static0_flash = (calibration_static_t *)FLASH_STATIC0_ADR;
static calibration_static_t *calibration_static1_flash = (calibration_static_t *)FLASH_STATIC1_ADR;

void get_calibration_static(void)
{

}

void get_calibration_dynamic(void)
{

}

void calibration_vout_init(void)
{

}

void calibration_iout_init(void)
{
}

void calibration_temperature_init(void)
{

}

void calibration_input_init(void)
{

}

void calibration_ramp_init(void)
{

}

int32_t cal_cnt2rvu(int32_t vcnt)
{

}

int32_t cal_rvu2cnt(int32_t rvu)
{

}

 /***************************************************************************************************
  *  Description:    Converts current sense counts to calibrated rvi*amps (1/RVI units)
  *  Preconditions:
  *  Inputs:         none
  *  Outputs:
  ***************************************************************************************************/
int32_t cal_cnt2rvi(int32_t icnt, int8_t channel)
{

}

 // only for the main IOUT
int32_t cal_rvi2cnt(int32_t amps)
{

}

void calculate_gain(int32_t x1, int32_t y1, int32_t x2, int32_t y2, cal_data_t *cal_data)
{

}

// value: voltage reference in RVU from the operator
// adc: adc reading
// return: 	1: calibration data (adc and reference is good)
//			0: invalid data (adc is not set correctly or value ref entered is not in range)
uint8_t vout_calibration_value_check(uint8_t point, int32_t value, int32_t adc)
{

	return 0;
}

// return: 	1: calibration data (adc and reference is good)
//			0: invalid data (adc is not set correctly or value ref entered is not in range)
uint8_t iout_calibration_value_check(uint8_t point, int32_t value, int32_t adc)
{

	return 0;
}

void calculate_crc8_static_calibration(calibration_static_t* calibration, uint8_t* crc)
{

}
