/******************************************************************************
 *  File: calibration.h
 *  Description:
 *  Author: Arnaud Hugron
 *  Date: 08/13/2015
 *****************************************************************************/

#ifndef CALIBRATION_H_
#define CALIBRATION_H_

#include "pmbus.h"
#include "analog.h"

// digital resolution for PMBus reading
#ifdef HPT5K_400V
#define RVU 						128u	// output voltage	1/128
#define RVI 						256u    // output current	1/256
#define VOUT_EXPONENT               7u      // !!! RVU = 2^VOUT_EXPONENT !!! see above
#define IOUT_EXPONENT               8u      // !!! RVI = 2^IOUT_EXPONENT !!!
#elif HPT5K_800V
#define RVU                         64u     // output voltage   1/64
#define RVI                         512u    // output current   1/512
#define VOUT_EXPONENT               6u      // !!! RVU = 2^VOUT_EXPONENT !!! see above
#define IOUT_EXPONENT               9u      // !!! RVI = 2^IOUT_EXPONENT !!!
#else
#define RVU                         256u    // output voltage   1/256
#define RVI                         32u     // output current   1/32
#define VOUT_EXPONENT               8u      // !!! RVU = 2^VOUT_EXPONENT !!! see above
#define IOUT_EXPONENT               5u      // !!! RVI = 2^IOUT_EXPONENT !!!
#endif

#define RVP 						2u		// output power		1/2
#define RVT							1u		// temperature		1/1
#define RVAC						1u		// Input AC			1/1

#define POUT_EXPONENT				1u		// !!! RVP = 2^POUT_EXPONENT !!!

#define VSET_DFLT            		256 //(MFR_VOUT_DEFAULT_RVU) TBD
#define VSET_MAX_DFLT        		(MFR_VOUT_MAX_RVU)
#define VSET_MIN_DFLT        		(MFR_VOUT_MIN_RVU)

#define VPFC_BUS_ADJ_DFLT        	128		// 0 <-> 255 (255 = 100%)	~788v
#define ISHARE_OFS_DFLT				128

//***************************************************************************************************
#define MFR_ID_DFLT			'X','P',' ','P','O','W','E','R','\0','\0','\0','\0','\0','\0','\0','\0','\0'
#define MFR_SERIAL_DFLT		'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F','\0'
#define MFR_REVISION		'0','0','0','2','\0'
#define MFR_LOCATION_DFLT	'U','S','A','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0','\0'
#define MFR_DATE_DFLT		'2','3','0','6','2','0','\0'		// YYMMDD

// maximum size for specific information in byte
#define	MFR_ID_LEN			16
#define	MFR_MODEL_LEN		32
#define MFR_REVISION_LEN	4
#define	MFR_SERIAL_LEN		16
#define	MFR_LOCATION_LEN	16
#define	MFR_DATE_LEN		6

#define GAIN_MULTIPLIER				16384ul	// 2^14
#define GAIN_MULTIPLIER_SHIFT		14u		// shift equivalent of GAIN_MULTIPLIER

// number of calibration point
#define NUM_IOUT_CAL_POINT			5u		// number of calibrated point for main output current sense
#define NUM_IOUTX_CAL_POINT			3u		// number of calibrated point for auxiliary output current sense (iout1, iout2 iout3)
#define NUM_VOUT_CAL_POINT			3u		// number of calibrated point for main output voltage sense
#define NUM_VPROG_CAL_POINT			3u		// number of calibrated point for vprog
#define NUM_IPROG_CAL_POINT			3u		// number of calibrated point for iprog

typedef struct
{
	int32_t	gain;
	int32_t	offset;
}cal_data_t;

typedef struct
{
    int32_t	adc;
    int32_t value;
}cal_analog_t;
//***************************************************************************************************

// flash signature for calibration in flash, validate data structure
#define CAL_STATIC_SIGNATURE	0x58ECu
#define CAL_DYNAMIC_SIGNATURE	0x695Au

//***************************************************************************************************
typedef struct calibration_header_t
{
    uint32_t    valide_data;        // // 0xFFFFFFFF if the flash data is valid with good signature, else not valid, check another sector
    uint8_t     version;
    uint8_t     crc8;
    uint16_t    signature;
} calibration_header_t;

// calibration structure mainly used for static data and calibration
// data is mainly set during ATE or manufacturing only not by user
typedef struct
{
	calibration_header_t    calibration_header;
	char_t		mfr_id[MFR_ID_LEN+1];
	char_t		mfr_model[MFR_MODEL_LEN+1];
	char_t		mfr_revision[MFR_REVISION_LEN+1];
	char_t		mfr_location[MFR_LOCATION_LEN+1];
	char_t		mfr_date[MFR_DATE_LEN+1];
	char_t		mfr_serial[MFR_SERIAL_LEN+1];

	uint16_t 	voltage_kp;
	uint16_t 	voltage_ki;
	uint16_t 	voltage_kd;

	uint16_t 	current_kp;
	uint16_t 	current_ki;
	uint16_t 	current_kd;

	uint16_t 	current_share_kp;
	uint16_t 	current_share_ki;
	uint16_t 	current_share_kd;

	uint16_t 	voltage_kalpha;
	uint16_t	voltage_kbeta_up;
	uint16_t	voltage_kbeta_down;
	uint16_t	voltage_kbeta_offset;

	uint16_t 	voltage_cot_kp;
	uint16_t 	voltage_cot_ki;
	uint16_t 	voltage_cot_kd;

	cal_analog_t vout_cal_point[NUM_VOUT_CAL_POINT];	// value is reference in RVU, adc is ADC register reading
	cal_data_t   vout_cal_gain[NUM_VOUT_CAL_POINT-1u];

	cal_analog_t iout_cal_point[NUM_IOUT_CAL_POINT];
	cal_data_t 	 iout_cal_gain[NUM_IOUT_CAL_POINT-1u];

	cal_analog_t iout_cal_point1[NUM_IOUTX_CAL_POINT];
	cal_data_t 	 iout_cal_gain1[NUM_IOUTX_CAL_POINT-1u];
	cal_analog_t iout_cal_point2[NUM_IOUTX_CAL_POINT];
	cal_data_t 	 iout_cal_gain2[NUM_IOUTX_CAL_POINT-1u];
	cal_analog_t iout_cal_point3[NUM_IOUTX_CAL_POINT];
	cal_data_t 	 iout_cal_gain3[NUM_IOUTX_CAL_POINT-1u];

	int16_t		 vprog_offset;
	int16_t		 iprog_offset;
	int16_t		 ishare_offset;

	int8_t		 pfc1_offset;
	int8_t		 pfc2_offset;
	int8_t		 pfc3_offset;

	cal_analog_t vprog_cal_point[NUM_VPROG_CAL_POINT];
	cal_data_t   vprog_cal_gain[NUM_VPROG_CAL_POINT-1u];
	cal_analog_t iprog_cal_point[NUM_IPROG_CAL_POINT];
	cal_data_t   iprog_cal_gain[NUM_IPROG_CAL_POINT-1u];
} calibration_static_t;

//***************************************************************************************************
// This data structure is used to record user settings. Can be changed by user if required
typedef struct
{
	calibration_header_t    calibration_header;
	pmbus_config_t 	pmbus_config;
	user_config_t	user_config;
	hardware_config_t hardware_config;
	uint8_t user_data00[USER_DATA_LEN+1];
	uint8_t user_data01[USER_DATA_LEN+1];
} calibration_dynamic_t;


// start EEPROM emulation from 0x38000 to 0x3FFFF
// use to store user and factory data
#define FLASH_STATIC_ADR				0x38000u
#define FLASH_DYNAMIC_FACTORY			0x39000u
#define FLASH_DYNAMIC_USER0_ADR			0x3A000u
#define FLASH_DYNAMIC_USER1_ADR			0x3B000u
#define FLASH_BLACKBOX0_ADR				0x3E000u
#define FLASH_BLACKBOX1_ADR				0x3F000u

#define FLASH_STATIC0_ADR               0x08E000  // FLASH_BANK0_SEC14
#define FLASH_STATIC1_ADR               0x08F000  // FLASH_BANK0_SEC15

//***************************************************************************************************
extern calibration_static_t calibration_static;
extern const calibration_static_t calibration_static_default;
extern calibration_dynamic_t calibration_dynamic_factory;
extern const calibration_dynamic_t calibration_dynamic_factory_default;
extern calibration_static_t calibration_static_ram;
extern calibration_dynamic_t calibration_dynamic_ram;

extern cal_data_t cal_vout_adj_10_percent_data;
extern cal_data_t cal_vout_adj_105_percent_data;
//***************************************************************************************************
void calibration_init(void);
void get_calibration_static(void);
void get_calibration_dynamic(void);
void calibration_vout_init(void);
void calibration_iout_init(void);
void calibration_temperature_init(void);
void calibration_input_init(void);
void calibration_ramp_init(void);
void calibration_init_dynamic_setting(void);

void calculate_gain(int32_t x1, int32_t y1, int32_t x2, int32_t y2, cal_data_t *cal_data);

uint8_t vout_calibration_value_check(uint8_t point, int32_t value, int32_t adc);
uint8_t iout_calibration_value_check(uint8_t point, int32_t value, int32_t adc);

// convert (V, I) into DSP count and vice versa to the respective resolution (RVU,RVI)
int32_t cal_cnt2rvu(int32_t vcnt);
int32_t cal_rvu2cnt(int32_t rvu);
int32_t cal_cnt2rvi(int32_t icnt, int8_t channel);
int32_t cal_rvi2cnt(int32_t amps);

void calculate_crc8_static_calibration(calibration_static_t* calibration, uint8_t* crc);

//***************************************************************************************************

#endif /* CALIBRATION_H_ */
