/******************************************************************************
 *  File: control.c
 *  Description:
 *  Author: Arnaud Hugron
 *  Date: 08/13/2015
 ****************************************************************************/
#include <stdint.h>
#include <stdlib.h>
#include "main.h"
#include "setup.h"
#include "control.h"
#include "control_soc.h"
#include "digital_io.h"
#include "flashInterface.h"
#include "state.h"
#include "preload.h"
#include "calibration.h"
#include "timer.h"
#include "primary.h"

/*lint -save -e931*/
	// Note 931: Both sides have side effects [MISRA 2012 Rule 1.3, required], [MISRA 2012 Rule 13.2, required]
control_t control;

int32_t ctrl_modulo;
uint8_t soft_start_index = 0;
uint8_t sequence_ctrl = SEQ_CTRL_IDLE;
uint8_t sequence_ctrl_state = SEQ_NONE;
uint16_t soft_start_step_timeout;
uint16_t soft_start_delay_ms;
uint8_t soft_start_step_retry;
uint8_t soft_start_step_max_retry;

#ifdef OPEN_LOOP
int32_t open_modulo;
#endif

#ifndef FIX_DEAD_TIME
uint32_t dead_time = DEAD_TIME;
#endif

// force PWM signals to 0
void mask_pwm_output(void)
{
    // do a force one shot trigger
    EPWM_forceTripZoneEvent(EPWM1_BASE, EPWM_TZ_FORCE_EVENT_OST);
    EPWM_forceTripZoneEvent(EPWM2_BASE, EPWM_TZ_FORCE_EVENT_OST);
    EPWM_forceTripZoneEvent(EPWM3_BASE, EPWM_TZ_FORCE_EVENT_OST);
}

// re-enable PWMs signal into pins
void unmask_pwm_output(void)
{
    if(status.bits.pwm_enable == 1u)
    {
        // force CBC event, it help synchronize PWM when we re-enable outputs
        EPWM_forceTripZoneEvent(EPWM1_BASE, EPWM_TZ_FORCE_EVENT_CBC);
        EPWM_forceTripZoneEvent(EPWM2_BASE, EPWM_TZ_FORCE_EVENT_CBC);
        EPWM_forceTripZoneEvent(EPWM3_BASE, EPWM_TZ_FORCE_EVENT_CBC);

        // clear OST to re enable output from previous OST
        EPWM_clearTripZoneFlag(EPWM1_BASE, EPWM_TZ_FLAG_OST);
        EPWM_clearTripZoneFlag(EPWM2_BASE, EPWM_TZ_FLAG_OST);
        EPWM_clearTripZoneFlag(EPWM3_BASE, EPWM_TZ_FLAG_OST);
    }
}

void control_init(void)
{
	// init voltage control parameter

	ctrl_modulo = MODULO_MIN_CTRL;

}

//***************************************************************************************************
void control_pwm_init(void)
{
    int32_t temp4;

	CONTROL_ISR_DISABLE();
	CONTROL_MASK_OUPUT();       // mask all LLC pwms outputs, disable PWMs output

    HWREGH(EPWM1_BASE + HRPWM_O_DBCTL) = 0x8C0B;    // ACTV_HIC, RED polarity is not inverted, FED polarity is inverted // 0x0B;
    HWREGH(EPWM2_BASE + HRPWM_O_DBCTL) = 0x8C0B;    // WARNING: seems to erase dead time FED register when enabling, put first !!!!!!!!!!!!!!
    HWREGH(EPWM3_BASE + HRPWM_O_DBCTL) = 0x8C0B;

    HWREG(EPWM1_BASE + HRPWM_O_DBFEDHR) = dead_time;  // dead time
    HWREG(EPWM2_BASE + HRPWM_O_DBFEDHR) = dead_time;  // dead time
    HWREG(EPWM3_BASE + HRPWM_O_DBFEDHR) = dead_time;  // dead time

    HWREGH(EPWM2_BASE + EPWM_O_TBCTL) |= EPWM_TBCTL_PHSEN;
    HWREGH(EPWM3_BASE + EPWM_O_TBCTL) |= EPWM_TBCTL_PHSEN;
    HWREG(EPWM2_BASE + HRPWM_O_TBPHS) = MODULO_DUTY_CYCLE_FREQ / 3u;    // TBD 6
    HWREG(EPWM3_BASE + HRPWM_O_TBPHS) = MODULO_DUTY_CYCLE_FREQ / 3u;

    temp4 = (MODULO_MIN_CTRL * -2) + 122683392;                             // calculate duty cycle
    HWREG(EPWM1_BASE + HRPWM_O_DBREDHR) = dead_time + (MODULO_DUTY_CYCLE_FREQ - ((MODULO_DUTY_CYCLE_FREQ - temp4) * 2)) * 2;
    HWREG(EPWM2_BASE + HRPWM_O_DBREDHR) = HWREG(EPWM1_BASE + HRPWM_O_DBREDHR);
    HWREG(EPWM3_BASE + HRPWM_O_DBREDHR) = HWREG(EPWM1_BASE + HRPWM_O_DBREDHR);

    GpioCtrlRegs.GPACTRL.all = 0x0A0A0A0A;                  // about 1us delay for GPIO0-GPIO31 with 6 samples qualification (PortA)

#ifdef OPEN_LOOP
    status.bits.pwm_enable = 1;
    open_modulo = MODULO_MIN_CTRL;
    CONTROL_ISR_ENABLE();           // enable PWMA channel 2 interrupt
    PWM_FAULT_OCP_ISR_CLEAR();      // clear OCP ISR flag
    PWM_FAULT_OCP_ISR_ENABLE();     // Enable OCP interrupt
    CONTROL_UNMASK_OUPUT();         // enable all LLC pwms outputs AT FORCE_OUT (MASTER RELOAD)
    CLEAR_OV_FAULT();               // clear OVP OST flag
#endif
    CONTROL_ISR_CLR_FLAG();
}

void control_loop_voltage_walkin_reset(void)
{
	voltage.int0 = MODULO_MIN_CTRL;
	voltage.Ki_active = voltage.Ki_walkin;
	voltage.Kp_active = voltage.Kp_walkin;
	system_flag.bits.voltage_walkin = 0;
}

void control_loop_current_walkin_reset(void)
{
	current.int0 = MODULO_MIN_CTRL;
	current.Ki_active = current.Ki_walkin;
	current.Kp_active = current.Kp_walkin;
}

#pragma CODE_SECTION(ocp_ISR, ".TI.ramfunc");
__interrupt void ocp_ISR(void)
{
   PWM_FAULT_OCP_ISR_CLEAR();
}

_TI_PROPRIETARY_PRAGMA("diag_push")
_TI_PROPRIETARY_PRAGMA("diag_suppress 112")

// !!! make sure that everything executed in this ISR is from RAM (table, function, ...)
// !!! we should not access ROM here or can have issue with FLASH update if we do that
// use "constant ON Time 7b.xlsx" file as formula reference
// use "constant ON Time 7d.xlsx" file as formula reference
#pragma CODE_SECTION(epwm1_ISR, ".TI.ramfunc");
__interrupt void epwm1_ISR(void)
{
	static int32_t cmpa_z1 = 0;
	static int32_t tpphs_z1 = 0;
	static int32_t red_z0 = 0;
	static int32_t cmpa_z0 = 0;
	static int32_t flag = 0;

	int32_t temp4;
	int32_t temp5;
	int32_t temp6;
	int32_t temp7;
	int32_t temp8;
	int32_t phase;

	DEBUG1_SIG_ON();

#ifdef OPEN_LOOP

set_open_control_loop:
	if(open_loop_state == 5)
	{
        open_loop_state = 6;
        open_modulo = pmbus_data.debugParam0;
        open_modulo <<= 16;
	}
	else if(open_loop_state == 6)
	{
        open_loop_state = 5;
        open_modulo = pmbus_data.debugParam1;
        open_modulo <<= 16;
	}
	else
	{
	    ctrl_modulo = open_modulo;
	}
#endif 	// OPEN_LOOP


// **************
// PWM generation
// **************
set_control_loop:
	if(ctrl_modulo >= MODULO_MAX_FREQ)
	{
	    DEBUG0_SIG_OFF();
        control.ctrl_freq_mode++;
	}
	else if(ctrl_modulo >= MODULO_MIN_CTRL_COT)
	{
	    // constant ON time 1.78us from 250KHz to 78947 Hz
        temp4 = (ctrl_modulo * (-5l)) + 188743680l;               // period

/////////////
#define TEST1
/////////////
#ifdef TEST1
        HWREG(EPWM1_BASE + HRPWM_O_TBPRDHR) = temp4;   // set period

        cmpa_z0 = (temp4 - MODULO_MAX_DUTY_CYCLE);      // period - 2us (ON time at 250KHz)
        HWREG(EPWM1_BASE + HRPWM_O_CMPA) = cmpa_z0;

        phase = temp4 / 6l;                             // 1/6 (60 phase shift)
        HWREG(EPWM2_BASE + HRPWM_O_TBPHS) = phase;      // Sync is from PWM1
        HWREG(EPWM3_BASE + HRPWM_O_TBPHS) = phase;      // Sync is from PWM2

        red_z0 = (dead_time>>1) + (temp4 - MODULO_MAX_FREQ); // * 2l;      // HALF cycle dead band clock,
        red_z0 *= 2l;
        HWREG(EPWM1_BASE + HRPWM_O_DBREDHR) = red_z0;
        HWREG(EPWM2_BASE + HRPWM_O_DBREDHR) = red_z0;
        HWREG(EPWM3_BASE + HRPWM_O_DBREDHR) = red_z0;

#else
        if((ctrl_modulo & 0xFFFF0000) == 0x1A00000)
        {
            if(flag == 0)
            {
                flag = 1;
                DEBUG1_SIG_OFF();
                temp7 = -(int32_t)calibration_static_ram.voltage_kbeta_offset<<16;
                temp5 =  (int32_t)calibration_static_ram.voltage_kbeta_up<<16;
                temp6 =  (int32_t)calibration_static_ram.voltage_kbeta_down<<16;
                temp8 = (int32_t)calibration_static_ram.voltage_kalpha<<16;
                DEBUG1_SIG_ON();
            }
            else
            {
                temp7 = 0;
                temp5 = 0;
                temp6 = 0;
                temp8 = 0;
            }
        }
        else
        {
            flag = 0;
            temp7 = 0;
            temp5 = 0;
            temp6 = 0;
            temp8 = 0;
        }
        temp4 = temp4 - temp6;
        HWREG(EPWM1_BASE + HRPWM_O_TBPRDHR) = temp4;          // set period

        cmpa_z0 = (temp4 - MODULO_MAX_DUTY_CYCLE);     // period - 2us (ON time at 250KHz)
        HWREG(EPWM1_BASE + HRPWM_O_CMPA) = cmpa_z0 - temp5;

        phase = temp4 / 6l; //calibration_static_ram.voltage_kbeta_offset; //phase_div;                                   // 1/6 (60 phase shift)
        phase = phase -  temp8;
        HWREG(EPWM2_BASE + HRPWM_O_TBPHS) = phase;
        HWREG(EPWM3_BASE + HRPWM_O_TBPHS) = phase; //<<1u;

        red_z0 = (dead_time>>1) + (temp4 - MODULO_MAX_FREQ); // * 2l;      // HALF cycle dead band clock,
        red_z0 = red_z0 + temp7;
        red_z0 *= 2;
        HWREG(EPWM1_BASE + HRPWM_O_DBREDHR) = red_z0;
        HWREG(EPWM2_BASE + HRPWM_O_DBREDHR) = red_z0;
        HWREG(EPWM3_BASE + HRPWM_O_DBREDHR) = red_z0;

#endif
	}
	else
	{
	}

    HWREGH(EPWM1_BASE + HRPWM_O_ETINTPS) = 2; // every other 2 cycles for interrupts

	CONTROL_SET_LDOK();												// set LDOK, update PWM registers at next reload
    DEBUG1_SIG_OFF();
	CONTROL_ISR_CLR_FLAG();											// clear interrupt flag
}
/*lint -restore */

