/******************************************************************************
 *  File: control.h
 *  Description:
 *  Author: Arnaud Hugron
 *  Date: 05/05/2023
 ****************************************************************************/

#ifndef CONTROL_H_
#define CONTROL_H_

#define FIX_DEAD_TIME 1

#define DSP_PWM_CLOCK               120000000l
#define FREQ_TO_COUNT(frequency)    (int32_t)((DSP_PWM_CLOCK*4/frequency)*16384l)

#define DEAD_TIME 0x300000          // Half cycle, already *2 (200ns)

#define MODULO_MAX_CTRL         FREQ_TO_COUNT(40000)        //0x0BB80000
#define MODULO_MIN_CTRL         0x00950000

#define MODULO_MAX_FREQ         FREQ_TO_COUNT(250000)       //0x01E00000
#define MODULO_MIN_CTRL_COT     0x01000000
#define MODULO_MAX_DUTY_CYCLE   (MODULO_MAX_FREQ>>1u)
#define MODULO_DUTY_CYCLE_FREQ  FREQ_TO_COUNT(75000)        //0x06400000

#define CONTROL_ISR_ENABLE()    Interrupt_enable(INT_EPWM1)
#define CONTROL_ISR_DISABLE()   Interrupt_disable(INT_EPWM1)
#define CONTROL_ISR_CLR_FLAG()  EPWM_clearEventTriggerInterruptFlag(EPWM1_BASE); Interrupt_clearACKGroup(INTERRUPT_ACK_GROUP3);

#define CONTROL_MASK_OUPUT()    mask_pwm_output()
#define CONTROL_UNMASK_OUPUT()  unmask_pwm_output()

#define CONTROL_CHANNEL1_ENABLE()
#define CONTROL_CHANNEL2_ENABLE()
#define CONTROL_CHANNEL3_ENABLE()

#define CONTROL_CLR_LDOK()
#define CONTROL_SET_LDOK()         EPWM_setGlobalLoadOneShotLatch(EPWM1_BASE)   // set LDOK, update PWM registers at next reload
#define CONTROL_SET_RUN()
#define CONTROL_CLR_RUN()

// PWM OCP faults
#define PWM_FAULT_OCP_ISR_ENABLE()      Interrupt_enable(OCP_I_XINT)
#define PWM_FAULT_OCP_ISR_DISABLE()     Interrupt_disable(OCP_I_XINT)
#define PWM_FAULT_OCP_ISR_CLEAR()       Interrupt_clearACKGroup(INTERRUPT_ACK_GROUP1)
#define CLEAR_OC_FAULT()                PWM_FAULT_OCP_ISR_CLEAR()
#define IS_OC_FAULT_PIN()               ((HWREG(GPIODATA_BASE+GPIO_O_GPADAT) & GPIO_GPADAT_GPIO10) == 0u) // state of the ocp fault pin

// OVP fault
#define EPWM_OVP_TZOST_FLAG            (EPWM_TZ_OST_FLAG_OST1 | EPWM_TZ_OST_FLAG_DCAEVT1 | EPWM_TZ_OST_FLAG_DCBEVT1)
#define EPWM_OVP_TZ_FLAG               (/*EPWM_TZ_FLAG_OST |*/  EPWM_TZ_FLAG_DCAEVT1 | EPWM_TZ_FLAG_DCBEVT1 | EPWM_TZ_INTERRUPT)    // TBD check

#define IS_OV_FAULT()               (EPwm1Regs.TZOSTFLG.all & (EPWM_TZ_FLAG_OST) != 0)  // OVP
#define IS_SW_OV_FAULT()            ((EPwm1Regs.TZOSTFLG.all & (EPWM_TZ_FLAG_DCAEVT1 | EPWM_TZ_FLAG_DCBEVT1)) != 0) //((EPwm1Regs.TZOSTFLG.all & (EPWM_TZ_FLAG_DCAEVT1 | EPWM_TZ_FLAG_DCAEVT2)) != 0)  // OVP
#define CLEAR_SW_OV_FAULT()         CLEAR_OV_FAULT()

inline void CLEAR_OV_FAULT(void)
{
    EALLOW;
    EPwm1Regs.TZOSTCLR.all = EPWM_OVP_TZOST_FLAG;
    EPwm2Regs.TZOSTCLR.all = EPWM_OVP_TZOST_FLAG;
    EPwm3Regs.TZOSTCLR.all = EPWM_OVP_TZOST_FLAG;
    EPwm1Regs.TZCLR.all = EPWM_OVP_TZ_FLAG;
    EPwm2Regs.TZCLR.all = EPWM_OVP_TZ_FLAG;
    EPwm3Regs.TZCLR.all = EPWM_OVP_TZ_FLAG;
    EDIS;
}

// Sequence for soft start or short jump
#define SEQ_NONE            0u      // Sequence done or if soft start, do SOC table
#define SEQ_SOC_STAGE1      1u      // first pulse of SOC (SOC soft start)
#define SEQ_SOC_STAGE2      2u      // set 123KHz pulse if we are in Duty Cycle Mode only
#define SEQ_SOC_STAGE2b     3u      // No Duty Cycle Mode, Skip higher frequency from SOC table, go only lower
#define SEQ_HIGH_TRANSIENT  4u      // when >= 2% undershoot, use transient table jump
#define SEQ_SHORT_FREQUENCY 5u      // jump to 148KHz after short detection (or after short 2 step jump)
#define SEQ_SHORT_3_STEPS   6u      // do a 3 step jump when short detected in Duty Cycle Mode, first pulse sequence
#define SEQ_SHORT_3_STEPS_2 7u      // second pulse of 3 step jump
#define SEQ_SHORT_3_STEPS_3 8u      // third pulse of 3 step jump
#define SEQ_SHORT_2_STEPS   9u      // do a 2 step jump when short detected in Frequency Mode, first pulse sequence
#define SEQ_SHORT_SKIP_STEPS 10u    // skip a pulse during OCP signal or short detection
#define SEQ_SHORT_SKIP_1_STEPS 11u  // skip another pulse during OCP signal or short detection, sometimes if OCP ISR occurs just before PWM reload, we do not skip the first pulse, add 1 more now
#define SEQ_SHORT_

#define SEQ_CTRL_IDLE   0x00u
#define SEQ_CTRL_ACTIVE 0xAAu

#define PERIOD_AFTER_S1         FREQ_TO_COUNT(123000)
#define PERIOD_AFTER_S3         FREQ_TO_COUNT(148000)   // Sequence 3, for short circuit jump

#define S6_PERIOD               FREQ_TO_COUNT(171585)

#ifdef HPT5K_LOW_LINE
// coefficient for transient HPL5K (Kinetis)
#define TRANSIENT_3_COEFF   6884        //7517
#define TRANSIENT_3_OFS     -97351441   //-111423635
#define TRANSIENT_2_COEFF   4344        //4521
#define TRANSIENT_2_OFS     -37825217   //-41214083
#define TRANSIENT_1_COEFF   2352
#define TRANSIENT_1_OFS     381404
#define TRANSIENT_0_COEFF   791
#define TRANSIENT_0_OFS     21995765

#define MODULO_MIN_FREQ_TRANSIENT    FREQ_TO_COUNT(59000)           // min freq jump during transient
#define MODULO_MAX_FREQ_TRANSIENT    FREQ_TO_COUNT(230000)

#else
// coefficient for transient for HPT5K

#ifdef KINETIS_DSP
// FOR KINETIS DSP as PWM frequency is 84MHz (120MHz for TI), equation calculated using "HPT HPL Family Design File.xlsx" -> "HPT Load Transient"
#define TRANSIENT_3_COEFF   7455
#define TRANSIENT_3_OFS     -111457351
#define TRANSIENT_2_COEFF   3993
#define TRANSIENT_2_OFS     -30308738
#define TRANSIENT_1_COEFF   2017
#define TRANSIENT_1_OFS     7580786
#define TRANSIENT_0_COEFF   863
#define TRANSIENT_0_OFS     23560194
#else
// FOR TI DSP as PWM frequency is 120MHz (84MHz for kinetis), equation calculated using "HPT HPL Family Design File.xlsx" -> "HPT Load Transient (TI DSP)"
#define TRANSIENT_3_COEFF       10650l
#define TRANSIENT_3_OFS         -159230544l
#define TRANSIENT_2_COEFF       5702l
#define TRANSIENT_2_OFS         -43258114l
#define TRANSIENT_1_COEFF       2882l
#define TRANSIENT_1_OFS         10814699l
#define TRANSIENT_0_COEFF       1233l
#define TRANSIENT_0_OFS         33654675l

#endif

#define MODULO_MIN_FREQ_TRANSIENT    FREQ_TO_COUNT(59000)           // min freq jump during transient
#define MODULO_MAX_FREQ_TRANSIENT    FREQ_TO_COUNT(210000)
#endif

typedef struct
{
    int32_t ctrl;
    int32_t voltage;
}soft_start_t;

extern uint8_t soft_start_index;
extern uint8_t sequence_ctrl;
extern uint8_t sequence_ctrl_state;

typedef struct
{
    uint16_t ctrl_modulo_filtered;
    uint16_t pri_comm_break_received;
    uint8_t ctrl_current_limit;
    uint8_t ctrl_freq_mode;
    uint8_t ctrl_cot_mode;
    uint8_t ctrl_dc_mode;
    uint8_t ctrl_undershoot;
    uint8_t ctrl_overshoot;
    uint8_t ctrl_short0;
    uint8_t ctrl_short1;
    uint8_t ctrl_short2;
    uint8_t ctrl_short3;
    uint8_t ctrl_short4;
    uint8_t ctrl_ocp_cnt;
    uint16_t ctrl_soc_step_delay_cnt;
    uint16_t ctrl_soc_step_delay_max;
    uint16_t ctrl_soc_step_timeout_max;
}control_t;

extern control_t control;
extern int32_t ctrl_modulo;
#ifdef OPEN_LOOP
extern int32_t open_modulo;
#endif

extern uint16_t soft_start_step_timeout;
extern uint16_t soft_start_delay_ms;
extern uint8_t soft_start_step_retry;
extern uint8_t soft_start_step_max_retry;

#define DEFAULT_SOC_STEP_DELAY 12u
#define DEFAULT_SOC_STEP_TIMEOUT 80u
#define DEFAULT_SOC_STEP_TIMEOUT_HIGH_GAIN 70u
#define DEFAULT_SOFT_START_STEP_MAX_RETRY 4u
#define DEFAULT_SOFT_START_STEP_COLD_MAX_RETRY 8u

//----- HPT5K 48V -----
#ifdef HPT5K_48V
#define VOLTAGE_WALKIN_Kp   8000
#define VOLTAGE_WALKIN_Ki   300
#define CURRENT_WALKIN_Kp   32
#define CURRENT_WALKIN_Ki   1
#define CURRENT_Kp          50
#define CURRENT_Ki          2
#define CURRENT_Kd          0

#ifdef HPT5K_LOW_LINE
#define VOLTAGE_Kp          6000
#define VOLTAGE_Ki          200
#define VOLTAGE_Kd          5

#define VOLTAGE_COT_Kp      20000
#define VOLTAGE_COT_Ki      200
#define VOLTAGE_COT_Kd      5

#define VOLTAGE_KAlpha      800     // Derivative low pass filter
#define VOLTAGE_KBeta_UP    0       // use during current transient, current derivative, improve transient response
#define VOLTAGE_KBeta_DOWN  0
#define VOLTAGE_KBeta_OFS   0

#else
#define VOLTAGE_Kp          6000
#define VOLTAGE_Ki          200
#define VOLTAGE_Kd          5

#define VOLTAGE_COT_Kp      20000
#define VOLTAGE_COT_Ki      200
#define VOLTAGE_COT_Kd      5

#define VOLTAGE_KAlpha      800     // Derivative low pass filter
#define VOLTAGE_KBeta_UP    0
#define VOLTAGE_KBeta_DOWN  0
#define VOLTAGE_KBeta_OFS   0
#endif

//----- HPT5K 60V -----
#elif HPT5K_60V
#define VOLTAGE_WALKIN_Kp   8000
#define VOLTAGE_WALKIN_Ki   300
#define CURRENT_WALKIN_Kp   32
#define CURRENT_WALKIN_Ki   1
#define CURRENT_Kp          50
#define CURRENT_Ki          2
#define CURRENT_Kd          0

#define VOLTAGE_Kp          6000
#define VOLTAGE_Ki          200
#define VOLTAGE_Kd          5

#define VOLTAGE_COT_Kp      20000
#define VOLTAGE_COT_Ki      200
#define VOLTAGE_COT_Kd      5

#define VOLTAGE_KAlpha      800     // Derivative low pass filter
#define VOLTAGE_KBeta_UP    0       // use during current transient, current derivative, improve transient response
#define VOLTAGE_KBeta_DOWN  0
#define VOLTAGE_KBeta_OFS   0

//----- HPT5K 80V -----
#elif HPT5K_80V
#define VOLTAGE_WALKIN_Kp   8000
#define VOLTAGE_WALKIN_Ki   200
#define VOLTAGE_Kp          3000
#define VOLTAGE_Ki          100
#define VOLTAGE_Kd          0
#define CURRENT_WALKIN_Kp   32
#define CURRENT_WALKIN_Ki   1
#define CURRENT_Kp          50
#define CURRENT_Ki          2
#define CURRENT_Kd          0
#define VOLTAGE_KAlpha      4               // Derivative low pass filter
#define VOLTAGE_KBeta       0 // 600        // use during current transient of 10% or more, improve transient response
#define VOLTAGE_COT_Kp      11000
#define VOLTAGE_COT_Ki      800
#define VOLTAGE_COT_Kd      150

//----- HPT5K 100V -----
#elif HPT5K_100V
#define VOLTAGE_WALKIN_Kp   8000
#define VOLTAGE_WALKIN_Ki   200
#define CURRENT_WALKIN_Kp   32
#define CURRENT_WALKIN_Ki   1
#define CURRENT_Kp          50
#define CURRENT_Ki          2
#define CURRENT_Kd          0

#define VOLTAGE_Kp          6000
#define VOLTAGE_Ki          200
#define VOLTAGE_Kd          5

#define VOLTAGE_COT_Kp      20000
#define VOLTAGE_COT_Ki      200
#define VOLTAGE_COT_Kd      5

#define VOLTAGE_KAlpha      800     // Derivative low pass filter
#define VOLTAGE_KBeta_UP    0       // use during current transient, current derivative, improve transient response
#define VOLTAGE_KBeta_DOWN  0
#define VOLTAGE_KBeta_OFS   0

//----- HPT5K 200V -----
#elif HPT5K_200V
#define VOLTAGE_WALKIN_Kp   8000
#define VOLTAGE_WALKIN_Ki   200
#define CURRENT_WALKIN_Kp   32
#define CURRENT_WALKIN_Ki   1
#define CURRENT_Kp          50
#define CURRENT_Ki          2
#define CURRENT_Kd          0

#define VOLTAGE_Kp          6000
#define VOLTAGE_Ki          200
#define VOLTAGE_Kd          5

#define VOLTAGE_COT_Kp      20000
#define VOLTAGE_COT_Ki      200
#define VOLTAGE_COT_Kd      5

#define VOLTAGE_KAlpha      800     // Derivative low pass filter
#define VOLTAGE_KBeta_UP    0       // use during current transient, current derivative, improve transient response
#define VOLTAGE_KBeta_DOWN  0
#define VOLTAGE_KBeta_OFS   0

//----- HPT5K 400V -----
#elif HPT5K_400V
#define VOLTAGE_WALKIN_Kp   8000
#define VOLTAGE_WALKIN_Ki   200
#define CURRENT_WALKIN_Kp   32
#define CURRENT_WALKIN_Ki   1
#define CURRENT_Kp          50
#define CURRENT_Ki          2
#define CURRENT_Kd          0

#define VOLTAGE_Kp          6000
#define VOLTAGE_Ki          200
#define VOLTAGE_Kd          5

#define VOLTAGE_COT_Kp      20000
#define VOLTAGE_COT_Ki      200
#define VOLTAGE_COT_Kd      5

#define VOLTAGE_KAlpha      800     // Derivative low pass filter
#define VOLTAGE_KBeta_UP    0       // use during current transient, current derivative, improve transient response
#define VOLTAGE_KBeta_DOWN  0
#define VOLTAGE_KBeta_OFS   0

//----- HPT5K 800V -----
#elif HPT5K_800V
#define VOLTAGE_WALKIN_Kp   8000
#define VOLTAGE_WALKIN_Ki   200
#define CURRENT_WALKIN_Kp   32
#define CURRENT_WALKIN_Ki   1
#define CURRENT_Kp          50
#define CURRENT_Ki          2
#define CURRENT_Kd          0

#define VOLTAGE_Kp          6000
#define VOLTAGE_Ki          200
#define VOLTAGE_Kd          5

#define VOLTAGE_COT_Kp      20000
#define VOLTAGE_COT_Ki      200
#define VOLTAGE_COT_Kd      5

#define VOLTAGE_KAlpha      800     // Derivative low pass filter
#define VOLTAGE_KBeta_UP    0       // use during current transient, current derivative, improve transient response
#define VOLTAGE_KBeta_DOWN  0
#define VOLTAGE_KBeta_OFS   0
#endif

#ifdef FIX_DEAD_TIME
#define dead_time DEAD_TIME         // Half cycle, *2
#else
extern uint32_t dead_time = DEAD_TIME;
#endif

//***************************************************************************************************
void control_init(void);
void control_pwm_init(void);
void control_loop_voltage_walkin_reset(void);
void control_loop_current_walkin_reset(void);
void mask_pwm_output(void);
void unmask_pwm_output(void);
//***************************************************************************************************

#define SOFT_START_STEPS 50u
extern soft_start_t soft_start_steps[SOFT_START_STEPS+1u];  // use for Safe Operating SoftStart

#endif /* CONTROL_H_ */
