/******************************************************************************
 *  File: fan.h
 *  Description:
 *  Author: Arnaud Hugron
 *  Date: 08/13/2015
 ****************************************************************************/
#ifndef FAN_H_
#define FAN_H_

#ifdef NXP_KINETIS
#ifdef HPT5K_LOW_PROFILE
#define	FAN_PWM_GAIN					790u / 100u	// 90% max PWM for low profile (else ripple current in caps too high)
#else
#define	FAN_PWM_GAIN					876u / 100u	// 100% is PWM3.VAL3 = 876 (counter up to 875), 8.76
#endif
#endif

#ifdef TI_C2000
#define FAN_PWM_GAIN                    820510ul     // div 4 clk TPRD = 1253 * 65536
#endif

#define FAN_RPM_TOO_LOW_THREHOLD 		10u  	// 1/32 RPM -> 320 RPM
#define FAN_FAIL_TIMEOUT_SEC 			15u		// 15s timeout fan failure
#define FAN_WARN_TIMEOUT_SEC 			5u		// 5s warning fan failure
#define FAN_SPEED_CTRL_MIN				15u		// %
#define FAN_ACOK_RESET_SEC				60u

typedef struct
{
    uint16_t fan1_speed;			// 1/32 RPM
    uint16_t fan2_speed;			// 1/32 RPM
    uint16_t fan3_speed;			// 1/32 RPM
    uint16_t fan4_speed;			// 1/32 RPM
    uint16_t fan_ctrl;           	// internal or command fan speed (0 to 100%)
    uint16_t fan_ctrl_pwm;          // actual fan speed %
    uint16_t fan_cold_hiccup_cnt;	// counter for very cold temperature to warm up primary NTC
    uint16_t fan_preload_offset;	// add fan speed if preload dissipate a lot of power
    uint16_t fan_acok_cnt;			// do not SD fan again with ACOK bad received until after at least 60s
    uint16_t fan_min_speed;			// min fan speed between all fans
	uint8_t  fan_select;
} fan_t;

extern fan_t fan;

void fan_init(void);
void fan_speed(void);
void fan_process(void);
void fan_pwm_process(void);

#ifdef TI_C2000
#define INIT_FAN_COUNTER()
#define SET_FAN_PWM(x)     (HWREG(EPWM4_BASE + HRPWM_O_CMPA) = (uint32_t)x * FAN_PWM_GAIN)

inline uint32_t GET_FAN_SPEED(void)
{
    uint32_t uTemp;

    // read fan speed tach pulse counter, Fan tach formula: RPM = 30 / T or RPM = 30 * F, where F is frequency of the fan tach pulse, speed is in 1/32 RPM
    uTemp =  (HWREG(ECAP1_BASE + ECAP_O_CAP1));
    if(uTemp != 0u)
    {
        uTemp = (16u * FAST_PERIPHERAL_CLOCK) / uTemp;  // FAN 2 pulses per RPM, CAPTURE event every 32 rising edges
    }
    else
    {
        uTemp = 0;
    }
    HWREG(ECAP1_BASE + ECAP_O_CAP1) = 0;                // reset capture register, for timeout or no fan
    HWREGH(ECAP1_BASE + ECAP_O_ECCTL2) |= ECAP_ECCTL2_CTRFILTRESET; // reset all other critical registers
    return uTemp;
}
#endif

#ifdef NXP_KINETIS
#define INIT_FAN_COUNTER()    {LPTMR_CSR &= ~0x01u;                            // disable LPTMR to clr CNR
                              LPTMR_CSR |= 0x01u;}                         // re-enable LPTMR, TCF is also clear, start counting fan pulse
#define SET_FAN_PWM(x)      { PWMA_MCTRL |= 0x80u;                                        // clear LDOK by setting CLDOK
                            PWMA_SM3VAL3 = (uint16_t)((uint32_t)x * FAN_PWM_GAIN);           // set duty cycle
                            PWMA_MCTRL |= 0x08u;}

inline uint32_t GET_FAN_SPEED(void)
{
    uint32_t uTemp;

    // read fan speed tach pulse counter, Fan tach formula: RPM = 30 / T or RPM = 30 * F, where F is frequency of the fan tach pulse, speed is in 1/32 RPM
    LPTMR_CNR = 0x0000;                                 // use LPTMR for FAN TACH, write 0 to capture the register else LPTMR_CNR will not be updated
    uTemp = (LPTMR_CNR * 30u)>>FAN_SHIFT_COEFF;         // LPTMR_CNR = number of positive pulse in 1s, give pulse frequency in HZ, accuracy +-1 pulse + crystal clock accuracy, in 1/32 RPM
    LPTMR_CSR &= (uint32_t)~0x01u;                      // disable LPTMR to clr CNR
    LPTMR_CSR |= (uint32_t)0x01u;                       // re-enable LPTMR, TCF is also clear
    return uTemp;
}
#endif


#endif /* FAN_H_ */
