/******************************************************************************
 *  File: flash.c
 *  Description: flash function
 *  Author: Arnaud Hugron
 *  Date: 08/13/2015
 *****************************************************************************/
#include "main.h"
#include "calibration.h"
#include "flashInterface.h"

#ifdef MODBUS_SUPPORT
#include "modbus_driver.h"
#endif

// Flash timing specification
// Erase Flash Sector: 14ms Typ, 114ms max
// Program LongWord: 65us Typ, 145us max
// Program Check: 45us max
// Read 1s Flash Sector: 60us

flash_flags_t flash_flags;

typedef enum
{
    E_FLASH_ERROR_NONE = 0,
    E_FLASH_ERROR_INIT,
    E_FLASH_ERROR_ERASE,
    E_FLASH_ERROR_ERASE_CHECK,
    E_FLASH_ERROR_PROGRAM,
    E_FLASH_ERROR_PROGRAM_CHECK,
} FlashInterface_ErrorType;

#define FLASH_ERROR_NONE            0
#define FLASH_ERROR_INIT            (1 << (E_FLASH_ERROR_INIT - 1))
#define FLASH_ERROR_ERASE           (1 << (E_FLASH_ERROR_ERASE - 1))
#define FLASH_ERROR_ERASE_CHECK     (1 << (E_FLASH_ERROR_ERASE_CHECK - 1))
#define FLASH_ERROR_PROGRAM         (1 << (E_FLASH_ERROR_PROGRAM - 1))
#define FLASH_ERROR_PROGRAM_CHECK   (1 << (E_FLASH_ERROR_PROGRAM_CHECK - 1))

static uint8_t Flash_Error;

void flash_init(void)
{
	flash_flags.all = 0;

    Fapi_initializeAPI(F021_CPU0_BASE_ADDRESS, 120);  // 120 = CPU_CLK_FREQ in MHz

    Fapi_StatusType oStatus;
    Fapi_FlashStatusType oFlashStatus;

    oStatus = Fapi_setActiveFlashBank(Fapi_FlashBank0);
    if(oStatus != Fapi_Status_Success)
    {
        Flash_Error |= FLASH_ERROR_INIT;
        return;
    }

    while (Fapi_checkFsmForReady() != Fapi_Status_FsmReady);

    oStatus = Fapi_issueAsyncCommand(Fapi_ClearMore);
    if(oStatus != Fapi_Status_Success)
    {
        Flash_Error |= FLASH_ERROR_INIT;
        return;
    }

    while (Fapi_checkFsmForReady() != Fapi_Status_FsmReady);

    oFlashStatus = Fapi_getFsmStatus();
    if (oFlashStatus != Fapi_Status_Success)
    {
        Flash_Error |= FLASH_ERROR_INIT;
        return;
    }
}

void flash_process(void)
{
	uint32_t addr;
	uint8_t crc = 0u;

#ifdef MODBUS_SUPPORT
	if(mb_modbus_state != MB_EV_READY)
	{
		return;
	}
#endif

	if ((status.bits.sec_calibration_mode == 1u) && (flash_flags.bits.write_cal_static == 1u) && (pmbus_data.write_protect0 == 0x00u))
	{
		flash_flags.bits.write_cal_static = 0;

		Flash_Error = FLASH_ERROR_NONE;

		// Calculate CRC
		crc = 0u;
		calculate_crc8_static_calibration(&calibration_static_ram, &crc);
		calibration_static_ram.calibration_header.crc8 = crc;

		// Write to first sector.
		set_dsp_normal_run();
		FlashEraseSector(FLASH_STATIC0_ADR);
		FlashVerifySection(FLASH_STATIC0_ADR, (uint16_t)(P_SECTOR_SIZE / FSL_FEATURE_FLASH_PFLASH_SECTION_CMD_ADDRESS_ALIGMENT));
		FlashProgram(FLASH_STATIC0_ADR, sizeof(calibration_static_ram), (uint8_t*)&calibration_static_ram);
		FlashProgramCheck(FLASH_STATIC0_ADR, sizeof(calibration_static_ram) >> 1, (uint8_t*)&calibration_static_ram, &addr);
		set_dsp_high_speed_run();

		// If there are no errors on flashing the first sector, store same calibration into the second sector.
		if (Flash_Error == FLASH_ERROR_NONE)
		{
		    set_dsp_normal_run();
            FlashEraseSector(FLASH_STATIC1_ADR);
            FlashVerifySection(FLASH_STATIC1_ADR, (uint16_t)(P_SECTOR_SIZE / FSL_FEATURE_FLASH_PFLASH_SECTION_CMD_ADDRESS_ALIGMENT));
            FlashProgram(FLASH_STATIC1_ADR, sizeof(calibration_static_ram), (uint8_t*)&calibration_static_ram);
            FlashProgramCheck(FLASH_STATIC1_ADR, sizeof(calibration_static_ram) >> 1, (uint8_t*)&calibration_static_ram, &addr);
            set_dsp_high_speed_run();

		} else {
		    // Do something here if the original fails.
		}

	}
	else
	{
		// nothing to do
	}
}

void FlashEraseSector(uint32_t dest)
{
    Fapi_StatusType oStatus;
    Fapi_FlashStatusType oFlashStatus;

    oStatus = Fapi_issueAsyncCommandWithAddress(Fapi_EraseSector, (uint32_t *)dest);
    if(oStatus != Fapi_Status_Success)
    {
        Flash_Error |= FLASH_ERROR_ERASE;
        return;
    }

    while (Fapi_checkFsmForReady() != Fapi_Status_FsmReady);

    oFlashStatus = Fapi_getFsmStatus();
    if (oFlashStatus != Fapi_Status_Success)
    {
        Flash_Error |= FLASH_ERROR_ERASE;
        return;
    }
}

// number is number of double phrases (16 bytes) -> FTFx_PSECTOR_SIZE / FSL_FEATURE_FLASH_PFLASH_SECTION_CMD_ADDRESS_ALIGMENT
void FlashVerifySection(uint32_t dest, uint16_t number)
{
    Fapi_StatusType oStatus;
    Fapi_FlashStatusType oFlashStatus;
    Fapi_FlashStatusWordType oFlashStatusWord;

    oStatus = Fapi_doBlankCheck((uint32_t *)dest, P_SECTOR_SIZE >> 1, &oFlashStatusWord);
    if(oStatus != Fapi_Status_Success)
    {
        pmbus_data.status_cml.bits.st_cml_mcu = 1;
        Flash_Error |= FLASH_ERROR_ERASE_CHECK;
        return;
    }

    while (Fapi_checkFsmForReady() != Fapi_Status_FsmReady);

    oFlashStatus = Fapi_getFsmStatus();
    if (oFlashStatus != Fapi_Status_Success)
    {
        Flash_Error |= FLASH_ERROR_ERASE_CHECK;
        pmbus_data.status_cml.bits.st_cml_mcu = 1;
        return;
    }
}

void FlashProgram(uint32_t dest, uint32_t size, uint8_t *pData)
{
    uint16_t commandSize;
    Fapi_StatusType oStatus;
    Fapi_FlashStatusType oFlashStatus;

    while (size != 0x0U)
    {

        commandSize = (size > 4) ? 4 : (uint16_t)size;

        oStatus = Fapi_issueProgrammingCommand((uint32 *)dest, pData, commandSize, 0, 0, Fapi_AutoEccGeneration);
        if(oStatus != Fapi_Status_Success)
        {
            Flash_Error |= FLASH_ERROR_PROGRAM;
            pmbus_data.status_cml.bits.st_cml_mcu = 1;
            return;
        }

        while (Fapi_checkFsmForReady() != Fapi_Status_FsmReady);

        oFlashStatus = Fapi_getFsmStatus();
        if (oFlashStatus != Fapi_Status_Success)
        {
            Flash_Error |= FLASH_ERROR_PROGRAM;
            pmbus_data.status_cml.bits.st_cml_mcu = 1;
            return;
        }

        dest += commandSize;
        size -= commandSize;
        pData = &pData[commandSize];
    }

	return;
}

void FlashProgram32bits(uint32_t dest, uint32_t Data)
{
    Fapi_StatusType oStatus;
    Fapi_FlashStatusType oFlashStatus;

    oStatus = Fapi_issueProgrammingCommand((uint32 *)dest, (uint16_t *)&Data, 2, 0, 0, Fapi_AutoEccGeneration);
    if(oStatus != Fapi_Status_Success)
    {
        Flash_Error |= FLASH_ERROR_PROGRAM;
        pmbus_data.status_cml.bits.st_cml_mcu = 1;
        return;
    }

    while (Fapi_checkFsmForReady() != Fapi_Status_FsmReady);

    oFlashStatus = Fapi_getFsmStatus();
    if (oFlashStatus != Fapi_Status_Success)
    {
        Flash_Error |= FLASH_ERROR_PROGRAM;
        pmbus_data.status_cml.bits.st_cml_mcu = 1;
        return;
    }

	return;
}

void FlashProgramCheck(uint32_t dest, uint32_t size, uint8_t *pExpectedData, uint32_t *pFailAddr)
{
    Fapi_FlashStatusType oFlashStatus;
    Fapi_FlashStatusWordType oFlashStatusWord;

    while(size > 0u)
    {

        oFlashStatus = Fapi_doVerify((uint32_t *)dest, 1, (uint32_t *)pExpectedData, &oFlashStatusWord);

        size--;

        if(oFlashStatus != Fapi_Status_Success)
        {
            Flash_Error |= FLASH_ERROR_PROGRAM_CHECK;
            pmbus_data.status_cml.bits.st_cml_mcu = 1;
            return;
        }
    }
}

void set_dsp_normal_run(void)
{
#ifdef NXP_KINETIS
	PWMA_SM0CTRL = 0x1400;			// alternative PWM cycle interrupt
	SIM_CLKDIV1 |= 0x10000000u;		// switch to normal speed for core 84MHz, need that for flash erase/programming
	SMC_PMCTRL &= 0x9Fu;			// Normal Speed Run mode (RUN)
	while(SMC_PMSTAT != 0x01u){};	// wait until RUN is set
#endif
}

void set_dsp_high_speed_run(void)
{
#ifdef NXP_KINETIS
	SMC_PMCTRL |= 0x60u;			// High Speed Run mode (HSRUN)
	while(SMC_PMSTAT != 0x80u){};	// wait until HSRUN is set
	SIM_CLKDIV1 &= 0x0FFFFFFFu;		// switch to high speed for core 168MHz
	PWMA_SM0CTRL = 0x0400;			// every PWM cycle interrupt reload
#endif
}
