/******************************************************************************
 *  File: pmbus.c
 *  Description: pmbus protocol function
 *  Author: Arnaud Hugron
 *  Date: 08/13/2015
 *****************************************************************************/
#include <math.h>
#include "main.h"

#include "version.h"
#include "pmbus_driver.h"
#include "pmbus.h"
#include "calibration.h"
#include "analog.h"
#include "control.h"
#include "state.h"
#include "primary.h"
#include "fan.h"
#include "timer.h"
#include "digital_io.h"
#include "pri_comm.h"
#include "restart.h"
#include "flashInterface.h"
#include "power.h"
#include "preload.h"

#include "bootmode.h"

/*__attribute__ ((section(".m_data_2")))*/ pmbus_data_t pmbus_data;
/*__attribute__ ((section(".m_data_2")))*/ pmbus_timer_t pmbus_timer;
/*__attribute__ ((section(".m_data_2")))*/ pmbus_real_mfr_config_t pmbus_real_mfr_config;
/*__attribute__ ((section(".m_data_2")))*/ system_reset_status_t system_reset_status;
/*                                      */
/*__attribute__ ((section(".m_data_2")))*/ shutdown_event_t shutdown_event;
/*__attribute__ ((section(".m_data_2")))*/ shutdown_event_t last_shutdown_event;
/*__attribute__ ((section(".m_data_2")))*/ shutdown_event_t shutdown_event_mask;
/*__attribute__ ((section(".m_data_2")))*/ status_sys_t status;
/*__attribute__ ((section(".m_data_2")))*/ system_flag_t system_flag;


static void pmbus_data_init(void);
static void pmbus_init_timer(void);

uint8_t pmbus_store_default_all(uint8_t *buffer, uint8_t size, uint8_t type);
uint8_t pmbus_clear_faults(uint8_t *buffer, uint8_t size, uint8_t type);
uint8_t pmbus_calibration(uint8_t *buffer, uint8_t size, uint8_t type);
uint8_t pmbus_status_cml(uint8_t *buffer, uint8_t size, uint8_t type);
uint8_t pmbus_write_protect(uint8_t *buffer, uint8_t size, uint8_t type);
uint8_t pmbus_status_byte(uint8_t *buffer, uint8_t size, uint8_t type);
uint8_t pmbus_status_word(uint8_t *buffer, uint8_t size, uint8_t type);
uint8_t pmbus_read_output(uint8_t *buffer, uint8_t size, uint8_t type);
uint8_t pmbus_read_condition(uint8_t *buffer, uint8_t size, uint8_t type);

uint8_t pmbus_voltage(uint8_t *buffer, uint8_t size, uint8_t type);
uint8_t pmbus_current(uint8_t *buffer, uint8_t size, uint8_t type);
uint8_t pmbus_temperature(uint8_t *buffer, uint8_t size, uint8_t type);
uint8_t pmbus_restart(uint8_t *buffer, uint8_t size, uint8_t type);
uint8_t pmbus_control(uint8_t *buffer, uint8_t size, uint8_t type);
uint8_t pmbus_ishare(uint8_t *buffer, uint8_t size, uint8_t type);
uint8_t pmbus_preload(uint8_t *buffer, uint8_t size, uint8_t type);
uint8_t pmbus_sec_comm_err(uint8_t *buffer, uint8_t size, uint8_t type);
uint8_t pmbus_pfc_data(uint8_t *buffer, uint8_t size, uint8_t type);
uint8_t pmbus_pfc_info(uint8_t *buffer, uint8_t size, uint8_t type);
uint8_t pmbus_runtime(uint8_t *buffer, uint8_t size, uint8_t type);

#define pmbus_set_invalid_command(i, i2c_buffer) { }

// setting below must be in linear format
#define VOUT_COMMAND_MIN        VSET_MIN_DFLT
#define VOUT_COMMAND_MAX        VSET_MAX_DFLT

#define VOUT_OV_FAULT_MIN     	(0 * RVU)
#define VOUT_OV_FAULT_MAX   	(MFR_VOUT_OVP * RVU)
#define VOUT_OV_WARN_MIN   		(0 * RVU)
#define VOUT_OV_WARN_MAX   		(MFR_VOUT_OVP * RVU)
#define VOUT_UV_FAULT_MIN    	(0 * RVU)
#define VOUT_UV_FAULT_MAX    	(MFR_VOUT_OVP * RVU)
#define VOUT_UV_WARN_MIN    	(0 * RVU)
#define VOUT_UV_WARN_MAX   		(MFR_VOUT_OVP * RVU)

#define IOUT_OC_LV_FAULT_MAX   	(MFR_VOUT_OVP * RVU)
#define IOUT_OC_LV_FAULT_MIN   	(0 * RVU)

#define IOUT_OC_FAULT_MAX    	(MFR_IOUT_MAX)
#define IOUT_OC_FAULT_MIN    	(0)
#define IOUT_OC_WARN_MAX    	(MFR_IOUT_MAX)
#define IOUT_OC_WARN_MIN     	(0)

#define OT_FAULT_MIN            (0)
#define OT_FAULT_MAX            (MFR_SEC_OTP)
#define OT_WARN_MIN             (0)
#define OT_WARN_MAX             (MFR_SEC_OTP)

#define OT_PRI_FAULT_MIN        (0)
#define OT_PRI_FAULT_MAX        (MFR_PRI_OTP)
#define OT_PRI_WARN_MIN         (0)
#define OT_PRI_WARN_MAX         (MFR_PRI_OTP)

#define VIN_OV_FAULT_MIN        (0)
#define VIN_OV_FAULT_MAX        (MFR_VIN_OVP)
#define VIN_OV_WARN_MIN         (0)
#define VIN_OV_WARN_MAX         (MFR_VIN_OVP)
#define VIN_UV_FAULT_MIN        (0)
#define VIN_UV_FAULT_MAX        (MFR_VIN_OVP)
#define VIN_UV_WARN_MIN         (0)
#define VIN_UV_WARN_MAX         (MFR_VIN_OVP)

//pmbus_limits shall follow the index of PMBUS_LIMITS_INDEX
const pmbus_limits_t pmbus_linear_limits[MAX_LINEAR_LIMITS_INDEX] =
{

};

#define CMD_CLASS_00_SIZE   4
#define CMD_CLASS_10_SIZE   7
#define CMD_CLASS_20_SIZE   2
#define CMD_CLASS_30_SIZE   14
#define CMD_CLASS_40_SIZE   16
#define CMD_CLASS_50_SIZE   11
#define CMD_CLASS_60_SIZE   0
#define CMD_CLASS_70_SIZE   16
#define CMD_CLASS_80_SIZE   16
#define CMD_CLASS_90_SIZE   15
#define CMD_CLASS_A0_SIZE   14
#define CMD_CLASS_B0_SIZE   16
#define CMD_CLASS_C0_SIZE   0
#define CMD_CLASS_D0_SIZE   16
#define CMD_CLASS_E0_SIZE   16
#define CMD_CLASS_F0_SIZE   16

static const command_t i2c_class_00[CMD_CLASS_00_SIZE] =
{

};

static const command_t i2c_class_10[CMD_CLASS_10_SIZE] =
{

};

static const command_t i2c_class_20[CMD_CLASS_20_SIZE] =
{

};

static const command_t i2c_class_30[CMD_CLASS_30_SIZE] =
{

};

static const command_t i2c_class_40[CMD_CLASS_40_SIZE] =
{

};

static const command_t i2c_class_50[CMD_CLASS_50_SIZE] =
{

};

static const command_t i2c_class_70[CMD_CLASS_70_SIZE] =
{

};

static const command_t i2c_class_80[CMD_CLASS_80_SIZE] =
{

};

static const command_t i2c_class_90[CMD_CLASS_90_SIZE] =
{

};

static const command_t i2c_class_A0[CMD_CLASS_A0_SIZE] =
{

};

static const command_t i2c_class_B0[CMD_CLASS_B0_SIZE] =
{

};

static const command_t i2c_class_D0[CMD_CLASS_D0_SIZE] =
{

};

static const command_t i2c_class_E0[CMD_CLASS_E0_SIZE] =
{

};

static const command_t i2c_class_F0[CMD_CLASS_F0_SIZE] =
{

};

// main matrix for PMBUS command
cmd_class pmbus_cmd[16] =
{

};

/*lint -save -e715 -e818*/
	// Symbol 'xx' (line xxx) not referenced [MISRA 2012 Rule 2.7, advisory]
	// Pointer parameter 'xx' (line xxx) could be declared as pointing to const [MISRA 2012 Rule 8.13, advisory]
// every 10ms
void check_vout(void)
{

}

// every 10ms
void check_iout(void)
{

}

// call every 50ms
void check_temperature(void)
{

}

void check_vin(void)
{

}

// get system reset status to see what cause the last reset
void check_system_reset_status(void)
{// TBD
//	system_reset_status.rcm_srs0 = RCM_SRS0;
//	system_reset_status.rcm_srs1 = RCM_SRS1;
//	system_reset_status.rcm_ssrs0 = RCM_SSRS0;
//	system_reset_status.rcm_ssrs1 = RCM_SSRS1;
//
//	system_reset_status.smc_pmstat = SMC_PMSTAT;	// power mode status
//	system_reset_status.pmc_lvdsc1 = PMC_LVDSC1;	// low voltage detect status
//	system_reset_status.pmc_lvdsc2 = PMC_LVDSC2;
//	system_reset_status.pmc_regsc = PMC_REGSC;
//
//	system_reset_status.wdog_rstcnt0 = (uint8_t)WDOG_RSTCNT;
//	system_reset_status.wdog_rstcnt1 = (uint8_t)(WDOG_RSTCNT>>8u);
}

void clear_system_reset_status(void)
{ // TBD
//	RCM_SSRS0 = 0xEF;		// write 1 to clear to all reset status
//	RCM_SSRS1 = 0x2E;		// write 1 to clear to all reset status
}

// called every 10ms in timer_process()
void pmbus_data_process(void)
{

}

uint8_t pmbus_write_protect(uint8_t *buffer, uint8_t size, uint8_t type)
{

}

uint8_t pmbus_restore_default_all(uint8_t *buffer, uint8_t size, uint8_t type)
{

}

uint8_t pmbus_store_default_all(uint8_t *buffer, uint8_t size, uint8_t type)
{

}

uint8_t pmbus_restore_user_all(uint8_t *buffer, uint8_t size, uint8_t type)
{

}

uint8_t pmbus_store_user_all(uint8_t *buffer, uint8_t size, uint8_t type)
{

}

uint8_t pmbus_status_byte(uint8_t *buffer, uint8_t size, uint8_t type)
{

}

uint8_t pmbus_status_word(uint8_t *buffer, uint8_t size, uint8_t type)
{

}

uint8_t pmbus_status_cml(uint8_t *buffer, uint8_t size, uint8_t type)
{

}

uint8_t pmbus_clear_faults(uint8_t *buffer, uint8_t size, uint8_t type)
{

}

#pragma CODE_SECTION(pmbus_operation, ".TI.ramfunc");
// every 250us
/*__attribute__ ((section(".mydata")))*/ void pmbus_operation(void)
{

}

uint8_t pmbus_read_output(uint8_t *buffer, uint8_t size, uint8_t type)
{

}

uint8_t pmbus_read_condition(uint8_t *buffer, uint8_t size, uint8_t type)
{

}

uint8_t pmbus_voltage(uint8_t *buffer, uint8_t size, uint8_t type)
{

}
uint8_t pmbus_current(uint8_t *buffer, uint8_t size, uint8_t type)
{

}
uint8_t pmbus_temperature(uint8_t *buffer, uint8_t size, uint8_t type)
{

}
uint8_t pmbus_restart(uint8_t *buffer, uint8_t size, uint8_t type)
{

}
uint8_t pmbus_control(uint8_t *buffer, uint8_t size, uint8_t type)
{

}

uint8_t pmbus_ishare(uint8_t *buffer, uint8_t size, uint8_t type)
{

}

uint8_t pmbus_preload(uint8_t *buffer, uint8_t size, uint8_t type)
{

}

uint8_t pmbus_runtime(uint8_t *buffer, uint8_t size, uint8_t type)
{

}


uint8_t pmbus_sec_comm_err(uint8_t *buffer, uint8_t size, uint8_t type)
{

}

uint8_t pmbus_pfc_data(uint8_t *buffer, uint8_t size, uint8_t type)
{

}

uint8_t pmbus_pfc_info(uint8_t *buffer, uint8_t size, uint8_t type)
{

}

#define CAL_VOUT_STRUCT_SIZE  ((sizeof(cal_analog_t) * NUM_VOUT_CAL_POINT) + (sizeof(cal_data_t) * (NUM_VOUT_CAL_POINT-1u)))
#define CAL_IOUT_STRUCT_SIZE  ((sizeof(cal_analog_t) * NUM_IOUT_CAL_POINT) + (sizeof(cal_data_t) * (NUM_IOUT_CAL_POINT-1u)))
#define CAL_IOUTX_STRUCT_SIZE  ((sizeof(cal_analog_t) * NUM_IOUTX_CAL_POINT) + (sizeof(cal_data_t) * (NUM_IOUTX_CAL_POINT-1u)))

/*lint -save -e9013 -e9049*/	// no 'else' at end of 'if ... else if' chain [MISRA 2012 Rule 15.7, required]
								// increment/decrement operation combined with other operation with side-effects [MISRA 2012 Rule 13.3, advisory]
uint8_t pmbus_calibration(uint8_t *buffer, uint8_t size, uint8_t type)
{

}
/*lint -restore */

// convert linear data into RVU, RVI, RVT or RVAC depending on the coeff
int16_t pm_linear_to_mfr(uint16_t value, int16_t coeff)
{

}

// for negative exponent only, but exponent parameter is a positive value
uint16_t pmbus_to_linear_format(uint16_t value, int8_t exponent)
{

}

/***************************************************************************************************
 *  Description:    Check if the configure data is within limits.
 *  Preconditions:
 *  Inputs:
 *  Outputs: 1: if config is valid else return 0
 ***************************************************************************************************/
uint8_t pmbus_config_limits_chk(uint8_t cmd, uint8_t config_data)
{

}

/***************************************************************************************************
 *  Description:    Check if write command is permitted
 *  Preconditions:
 *  Inputs:
 *  Outputs:
 ***************************************************************************************************/
uint8_t pmbus_write_access(uint8_t cmd)
{

}

/***************************************************************************************************
 *  Description:    set user OVP limit using internal CMP#2 with internal DAC
 *  Preconditions:
 *  Inputs: OVP in linear format
 *  Outputs: none
 ***************************************************************************************************/
void set_user_ovp(uint16_t ovp_linear)
{
	// set OVP DAC (comparator, user OVP)

}

/***************************************************************************************************
 *  Description:    Check if the linear-format write data is within limits. If yes, updated the pmbus_data data accordingly and return TRUE.
 *  Preconditions:
 *  Inputs:
 *  Outputs: 0: not ok, 1: ok
 ***************************************************************************************************/
uint8_t pmbus_linear_limits_chk(uint16_t linear_data, uint8_t limits_index)
{

}

// pmbus_buffer[0] : command code to read
// response will be copy into pmbus_buffer[0]
// return the number of data to respond
/*__attribute__ ((section(".mydata")))*/ uint8_t pmbus_cmd_handler_read(uint8_t *pmbus_buffer, uint8_t pmbus_count)
{

}

// pmbus_buffer[0] : command code to read
// pmbus_buffer[1] : size for block packet or data for byte registers or LSB for WORD register
// pmbus_buffer[2] : start of data for block packet or MSB for word register
void pmbus_cmd_handler_write(uint8_t *pmbus_buffer, uint8_t pmbus_count)
{

}

static void pmbus_data_init(void)
{
	pmbus_data.vin = 0;
	pmbus_data.vout = 0;
	pmbus_data.iout = 0;
	pmbus_data.vout_mode = VOUT_MODE;

	pmbus_data.fan1_speed = 0;
	pmbus_data.fan2_speed = 0;
	pmbus_data.fan3_speed = 0;
	pmbus_data.fan4_speed = 0;

	pmbus_data.temperature_pfc1 = (uint16_t)pfc_data1.temperature_c;
	pmbus_data.temperature_pfc2 = (uint16_t)pfc_data1.temperature_c;
	pmbus_data.temperature_pfc3 = (uint16_t)pfc_data1.temperature_c;

	pmbus_data.temperature_sec1 = (uint16_t)temperature.temp1_c;
	pmbus_data.temperature_sec2 = (uint16_t)temperature.temp2_c;
	pmbus_data.temperature_sec3 = (uint16_t)temperature.temp3_c;

	pmbus_data.temperature_max_sec = (uint16_t)temperature.temp1_c;
	pmbus_data.temperature_max_pri = (uint16_t)pfc_data1.temperature_c;

    pmbus_data.write_protect0 = 0x80;			// write protected by default
    pmbus_data.on_off_config = 0x1C;

    pmbus_data.fan_config_1_2 = FAN_CONFIG_1_2_DEFAULT;
    pmbus_data.fan_config_3_4 = FAN_CONFIG_3_4_DEFAULT;

    pmbus_data.firmware_revision = FRW_VERSION;

    pmbus_data.vout_ov_fault_response = VOUT_OV_FAULT_RESPONSE_DEFAULT;

	pmbus_data.debug0 = 0;
	pmbus_data.debug1 = 0;
	pmbus_data.debug2 = 0;
	pmbus_data.debug3 = 0;
	pmbus_data.debugParam0 = 0;
	pmbus_data.debugParam1 = 0;

    shutdown_event_mask.all = 0;
	shutdown_event.all = 0;
	shutdown_event.bits.ac1_bad = 1;			// Start out with AC bad until we know for sure
	shutdown_event.bits.ac2_bad = 1;
	shutdown_event.bits.ac3_bad = 1;
	shutdown_event.bits.ac_imbalance = 1;

	shutdown_event.bits.pfc1_bad = 1;			// PFC not good by default until we get the go ahead
	shutdown_event.bits.pfc2_bad = 1;			// PFC not good by default until we get the go ahead
	shutdown_event.bits.pfc3_bad = 1;			// PFC not good by default until we get the go ahead

//	shutdown_event.bits.remote_off = 1;			// remote on/off, off by default until we check it.
	shutdown_event.bits.pmbus_off = 1;			// pmbus on/off, off by default until we check it.

	if (calibration_dynamic_ram.pmbus_config.operation == 0x00u)			// check if we need to start or not the power supply by default depending on pmbus operation register
	{
		shutdown_event.bits.pmbus_off = 1;
	}

	last_shutdown_event.all = shutdown_event.all;

	status.all = 0;
	system_flag.all = 0;
	system_flag.bits.sf_clear_fault_flag0 = 1;
	system_flag.bits.fast_pri_comm = 1;		// speed up comm at the beginning, need to turn ON PFC as fast as possible
	system_flag.bits.temp_cold = 1;			// temp cold by default

    pmbus_init_timer();
}

void pmbus_init(void)
{
	pmbus_data_init();
	pmbus_drv_init();
}

static void pmbus_init_timer(void)
{
	TIMER_ARM(pmbus_timer.power_up_ms, POWER_UP_MS);

	TIMER_ARM(pmbus_timer.vout_ovfd_ms, VOUT_OVFD_MS);
	TIMER_ARM(pmbus_timer.vout_ovwd_ms, VOUT_OVWD_MS);
	TIMER_ARM(pmbus_timer.vout_uvwd_ms, pmbus_timer.vout_uv_max_delay_ms);
	TIMER_ARM(pmbus_timer.vout_uvfd_ms, pmbus_timer.vout_uv_max_delay_ms);

	TIMER_ARM(pmbus_timer.iout_ocwd_ms, pmbus_timer.iout_oc_max_delay_ms);
	TIMER_ARM(pmbus_timer.iout_ocfd_ms, pmbus_timer.iout_oc_max_delay_ms);

	pmbus_timer.ot_fault_pfc1_delay = 0;
	pmbus_timer.ot_fault_pfc2_delay = 0;
	pmbus_timer.ot_fault_pfc3_delay = 0;

	pmbus_timer.ot_warn_pfc1_delay = 0;
	pmbus_timer.ot_warn_pfc2_delay = 0;
	pmbus_timer.ot_warn_pfc3_delay = 0;

	pmbus_timer.ot_fault_sec1_delay = 0;
	pmbus_timer.ot_fault_sec2_delay = 0;
	pmbus_timer.ot_fault_sec3_delay = 0;

	pmbus_timer.ot_warn_sec1_delay = 0;
	pmbus_timer.ot_warn_sec2_delay = 0;
	pmbus_timer.ot_warn_sec3_delay = 0;

	pmbus_timer.ot_fault_pri_delay = 0;
	pmbus_timer.timer_50ms = 0;

	TIMER_ARM(pmbus_timer.soft_start_ocp_disable_delay_ms, SOFT_START_OCP_DISABLE_MS);
	TIMER_ARM(pmbus_timer.pmbus_timeout, 35);
}

#pragma CODE_SECTION(pmbus_timer_process, ".TI.ramfunc");
// executed every 1ms
/*__attribute__ ((section(".mydata")))*/ void pmbus_timer_process()
{
	if(pmbus_timer.power_up_ms != 0u)  {pmbus_timer.power_up_ms--;}
	if(pmbus_timer.vout_ovfd_ms != 0u) {pmbus_timer.vout_ovfd_ms--;}
	if(pmbus_timer.vout_ovwd_ms != 0u) {pmbus_timer.vout_ovwd_ms--;}
	if(pmbus_timer.vout_uvfd_ms != 0u) {pmbus_timer.vout_uvfd_ms--;}
	if(pmbus_timer.vout_uvwd_ms != 0u) {pmbus_timer.vout_uvwd_ms--;}
	if(pmbus_timer.iout_ocfd_ms != 0u) {pmbus_timer.iout_ocfd_ms--;}
	if(pmbus_timer.iout_ocwd_ms != 0u) {pmbus_timer.iout_ocwd_ms--;}
	if(pmbus_timer.soft_start_ocp_disable_delay_ms != 0u) {pmbus_timer.soft_start_ocp_disable_delay_ms--;}
	if(preload.preload_max_diff_timer < 60000u) {preload.preload_max_diff_timer++;} 	// reset delay for max difference between vout and u_ref (preload cannot pull down vout)
	if(soft_start_delay_ms != 0u) {soft_start_delay_ms--;}
#ifdef CANBUS_SUPPORT
	if(co_sdo.timeoutTimer < 65500u) {co_sdo.timeoutTimer++;}
#endif
	if(pmbus_timer.pmbus_timeout != 0u) {pmbus_timer.pmbus_timeout--;}
}

uint32_t RealToPmbusLinear(float_t Data)
{

}

float_t PmbusLinearToReal(uint16_t Data)
{

}
