/******************************************************************************
 *  File: pmbus.h
 *  Description: pmbus protocol function
 *  Author: Arnaud Hugron
 *  Date: 08/13/2015
 *****************************************************************************/
#ifndef PMBUS_H_
#define PMBUS_H_

#include "main.h"

typedef float float_t;

extern void pmbus_timer_process(void);
extern void pmbus_data_process(void);
extern void pmbus_init(void);
extern void pmbus_cmd_handler_write(uint8_t *pmbus_buffer, uint8_t pmbus_count);
extern uint8_t pmbus_cmd_handler_read(uint8_t *pmbus_buffer, uint8_t pmbus_count);
extern int16_t pm_linear_to_mfr(uint16_t value, int16_t coeff);
extern uint16_t pmbus_to_linear_format(uint16_t value, int8_t exponent);
extern uint8_t pmbus_write_access(uint8_t cmd);
extern uint8_t pmbus_linear_limits_chk(uint16_t linear_data, uint8_t limits_index);
extern uint8_t pmbus_config_limits_chk(uint8_t cmd, uint8_t config_data);
extern void check_system_reset_status(void);
extern void clear_system_reset_status(void);
extern uint32_t RealToPmbusLinear(float_t Data);
extern float_t PmbusLinearToReal(uint16_t Data);
extern void set_user_ovp(uint16_t ovp_linear);
extern void check_vout(void);
extern void check_iout(void);
extern void check_temperature(void);
extern void check_vin(void);
extern void pmbus_operation(void);
extern uint8_t pmbus_store_user_all(uint8_t *buffer, uint8_t size, uint8_t type);
extern uint8_t pmbus_restore_user_all(uint8_t *buffer, uint8_t size, uint8_t type);
extern uint8_t pmbus_restore_default_all(uint8_t *buffer, uint8_t size, uint8_t type);
uint8_t pmbus_calibration(uint8_t *buffer, uint8_t size, uint8_t type);

/*****************************************************************************
* Configure the linear data coefficients in the DSP format
******************************************************************************/
#define FAN_SHIFT_COEFF 		5u
#define FAN_EXPONENT_MASK   	0x2800u  		// N=5, need to multiply data by 32 to get speed in RPM
#define FAN_COMMAND_MIN         0
#define FAN_COMMAND_MAX         100

#define PMBUS_BASE_SLAVE_ADDR	0xB0			// I2C base address

#define PMBUS_REV       0x21

#ifdef HPT5K_400V
#define VOUT_MODE       0x19					// linear with exponent N=-7 (128)
#elif HPT5K_800V
#define VOUT_MODE       0x1A                    // linear with exponent N=-6 (64)
#else
#define VOUT_MODE       0x18                    // linear with exponent N=-8 (256)
#endif

#define MFR_TAMB_MAX	50
#define MFR_TAMB_MIN	0x07EC					// -20 C

#define POWER_UP_MS 	(3000)
#define VOUT_OVFD_MS 	(50)
#define VOUT_OVWD_MS 	(30)

#define OT_FAULT_DELAY	200u	// 50ms step
#define OT_WARN_DELAY	60u		// 50ms step

#define VOUT_HYSTESIS	(RVU_TO_VSENSE_COUNT((RVU*1))/10)	// 0.1

#define RSP_TYPE_MASK				(0xC0u)
#define RSP_RETRY_MASK				(0x38u)
#define RSP_DELAY_MASK				(0x07u)

#define IOUT_RSP_OC_CONTINIOUS		(0x00u)
#define IOUT_RSP_OCLV  				(0x40u)
#define IOUT_RSP_OC_DELAY			(0x80u)
#define IOUT_RSP_OC_SD				(0xC0u)

#define VOUT_RSP_UV_CONTINIOUS		(0x00u)
#define VOUT_RSP_UV_DELAY			(0x40u)
#define VOUT_RSP_UV_SD				(0x80u)
#define VOUT_RSP_UV_SD1				(0xC0u)

#define VOUT_UV_LATCH_RSP  			(0x80u)
#define VOUT_UV_RESTART_RSP 		(0xC0u)

#define OT_LATCH_RSP       			(0x80u)
#define OT_RESTART_RSP      		(0xC0u)
#define OT_SEC_HYSTERESIS   		15		// C
#define OT_PRI_HYSTERESIS   		12		// C

#define SOFT_START_OCP_DISABLE_MS 	(100)

#define USER_DATA_LEN				16

#define UART_BAUDRATE_DEFAULT	19200	// baudrate
#define UART_STOP_BIT_DEFAULT	0		// stop bit count 0:1bits, 1:2bits
#define UART_PARITY_DEFAULT		2		// Parity 0:none, 2:even, 3:odd
#define UART_DATA_BIT_DEFAULT	0		// 8 bits data

#define REMOTE_ON_COUNTER 4
#define REMOTE_OFF_COUNTER 6

/*****************************************************************************
* PMBUS Command Type
******************************************************************************/
#define TYPE_NA         0xFFu    					// Not supported
#define TYPE_RD         0x80u
#define TYPE_WR         0x40u

#define TYPE_RD32       (uint8_t)(0x30u | TYPE_RD)          // read DWORD (32bits)
#define TYPE_RW         (uint8_t)(TYPE_RD | TYPE_WR)
#define TYPE_EE         (uint8_t)(0x20u | TYPE_RW)  		// EEPROM RW
#define TYPE_F_RD       (uint8_t)(0x10u | TYPE_RD)    		// function call, read only
#define TYPE_F_WR       (uint8_t)(0x10u | TYPE_WR)    		// function call, write only
#define TYPE_F_RW      	(uint8_t)(0x10u | TYPE_RW)    		// function call, RW
#define TYPE_RD_MFGW  	(uint8_t)(0x08u | TYPE_RW)			// block format that can be writable only during manufacturing or calibration
#define TYPE_CONST		0x30u

/*****************************************************************************
* PMBus special command handling
******************************************************************************/
#define VOUT_OV_FAULT_LIMIT_DEFAULT 	(MFR_VOUT_OVP * RVU)
#define VOUT_OV_WARN_LIMIT_DEFAULT		(MFR_VOUT_OVPW * RVU)

#define IOUT_OC_FAULT_LIMIT_DEFAULT		MAX_HL_CAPACITY_IN_AMPS

#ifdef HPT5K_200V
#define IOUT_OC_WARN_LIMIT_DEFAULT		(MAX_HL_CAPACITY_IN_AMPS-1)
#else
#define IOUT_OC_WARN_LIMIT_DEFAULT		(MAX_HL_CAPACITY_IN_AMPS-2)
#endif

#define IOUT_OC_LV_FAULT_LIMIT_DEFAULT	(0*RVU)

#define OT_SEC_FAULT_LIMIT_DEFAULT		MFR_SEC_OTP
#define OT_SEC_WARN_LIMIT_DEFAULT		MFR_SEC_OTP_WARNING
#define OT_PRI_FAULT_LIMIT_DEFAULT		MFR_PRI_OTP
#define OT_PRI_WARN_LIMIT_DEFAULT		MFR_PRI_OTP_WARNING

#define OPERATION_DEFAULT               0x80

#ifdef HPT5K_LOW_PROFILE
#define FAN_CONFIG_1_2_DEFAULT  		0x99
#define FAN_CONFIG_3_4_DEFAULT  		0x90
#else
#define FAN_CONFIG_1_2_DEFAULT  		0x90
#define FAN_CONFIG_3_4_DEFAULT  		0x00
#endif

#define VIN_OV_FAULT_LIMIT_DEFAULT		MFR_VIN_OVP
#define VIN_OV_WARN_LIMIT_DEFAULT		(MFR_VIN_OVP-4)
#define VIN_UV_WARN_LIMIT_DEFAULT		(MFR_VIN_UVP+5)
#define VIN_UV_FAULT_LIMIT_DEFAULT		MFR_VIN_UVP

#define VOUT_OV_FAULT_RESPONSE_DEFAULT  0x80	// SD, no retry
#define VOUT_UV_FAULT_RESPONSE_DEFAULT	0x00	// continue, no SD
#define IOUT_OC_FAULT_RESPONSE_DEFAULT	0x00	// continue, no SD, constant current limit
#define OT_FAULT_RESPONSE_DEFAULT		0xC0	// SD delay, retry when fault is gone
#define VIN_OV_FAULT_RESPONSE_DEFAULT	0xC0	// SD delay, retry when fault is gone
#define VIN_UV_FAULT_RESPONSE_DEFAULT	0x77	// SD delay, retry 6 times
#define IOUTX_FAULT_RESPONSE_DEFAULT	0x77	// SD delay, retry 6 times, restart after 70ms

#define USER_CONFIG_DEFAULT				0x0304	// preload not disable, ishare signal, current walkin, active signal level (ACOK, DCOK, TEMPOK, FANFAIL and remote inhibit), SYNC enable
												// remote inhibit is active high by default
#define HARDWARE_CONFIG_DEFAULT			0x00	// RS485 full duplex(1), modbus protocol(0)

#ifdef HPT5K_400V
#define RAMP_VOLTAGE_MIN				50		// ramp time ms
#define RAMP_VOLTAGE_MAX                5000    // ramp time ms
#define RAMP_VOLTAGE_FAULT_MIN          500     // when unit shutdown due to fault, we use a min of 400ms ramp down

#define RAMP_CURRENT_MIN                20      // ramp time ms
#define RAMP_CURRENT_MAX                20000   // ramp time ms

#elif HPT5K_800V
#define RAMP_VOLTAGE_MIN                100     // ramp time ms
#define RAMP_VOLTAGE_MAX                5000    // ramp time ms
#define RAMP_VOLTAGE_FAULT_MIN          500     // when unit shutdown due to fault, we use a min of 400ms ramp down

#define RAMP_CURRENT_MIN                20      // ramp time ms
#define RAMP_CURRENT_MAX                20000   // ramp time ms

#else
#define RAMP_VOLTAGE_MIN                35      // ramp time ms
#define RAMP_VOLTAGE_MAX                5000    // ramp time ms
#define RAMP_VOLTAGE_FAULT_MIN          400     // when unit shutdown due to fault, we use a min of 400ms ramp down

#define RAMP_CURRENT_MIN                20      // ramp time ms
#define RAMP_CURRENT_MAX                20000   // ramp time ms
#endif

// 0 byte size command, needed for CANOpen as there is no object type with 0 size data for .eds file
#define CLEAR_FAULTS				0x03u
#define STORE_DEFAULT_ALL			0x11u
#define RESTORE_DEFAULT_ALL			0x12u
#define STORE_USER_ALL				0x15u
#define RESTORE_USER_ALL			0x16u

// Status Word
typedef union
{
	uint16_t all;
	uint8_t byte[2];
	struct
	{
		unsigned    sts_none        :1;
		unsigned    sts_cml         :1;
		unsigned    sts_temperature :1;
		unsigned    sts_vin_uv      :1;

		unsigned    sts_iout_oc     :1;
		unsigned    sts_vout_ov     :1;
		unsigned    sts_off			:1;
		unsigned    sts_busy 		:1;

		unsigned    sts_unknown     :1;
		unsigned    sts_other       :1;
		unsigned    sts_fan 		:1;
		unsigned    sts_npower_good :1;

		unsigned    sts_mfr     	:1;
		unsigned    sts_input     	:1;
		unsigned    sts_iout		:1;
		unsigned    sts_vout 		:1;
	} bits;
} pmbus_status_word_t;

// Status VOUT
typedef union
{
	uint8_t all;
	struct
	{
		unsigned    st_vout_rfu     :1;
		unsigned    st_vout_toff    :1;
		unsigned    st_vout_ton		:1;
		unsigned    st_vout_max 	:1;

		unsigned    st_vout_uv     	:1;
		unsigned    st_vout_uvw     :1;
		unsigned    st_vout_ovw		:1;
		unsigned    st_vout_ov 		:1;
	} bits;
} pmbus_status_vout_t;

// Status IOUT
typedef union
{
	uint8_t all;
	struct
	{
		unsigned    st_pout_opw       :1;
		unsigned    st_pout_op        :1;
		unsigned    st_iout_pwr_limit :1;
		unsigned    st_iout_share 	  :1;

		unsigned    st_iout_uc     	  :1;
		unsigned    st_iout_ocw       :1;
		unsigned    st_iout_oclv	  :1;
		unsigned    st_iout_oc 		  :1;
	} bits;
} pmbus_status_iout_t;

// Status Input
typedef union
{
	uint8_t all;
	struct
	{
		unsigned    st_pin_op   	:1;
		unsigned    st_iin_ocw      :1;
		unsigned    st_iin_oc 		:1;
		unsigned    st_inp_off 	  	:1;

		unsigned    st_vin_uv       :1;
		unsigned    st_vin_uvw      :1;
		unsigned    st_vin_ovw	  	:1;
		unsigned    st_vin_ov  	    :1;
	} bits;
} pmbus_status_input_t;

// Status Temperature
typedef union
{
	uint8_t all;
	struct
	{
		unsigned    st_preload_otp	:1;
		unsigned    st_preload_otp1 :1;
		unsigned    st_temp_rfu2 	:1;
		unsigned    st_temp_rfu1 	:1;

		unsigned    st_temp_ut      :1;
		unsigned    st_temp_utw     :1;
		unsigned    st_temp_otw	  	:1;
		unsigned    st_temp_ot  	:1;
	} bits;
} pmbus_status_temperature_t;

// Status CML
typedef union
{
	uint8_t all;
	struct
	{
		unsigned    st_cml_logic	:1;
		unsigned    st_cml_comm     :1;
		unsigned    st_cml_rfu 		:1;
		unsigned    st_cml_mcu 	  	:1;

		unsigned    st_cml_memory   :1;
		unsigned    st_cml_err_pec  :1;
		unsigned    st_cml_inv_data	:1;
		unsigned    st_cml_inv_cmd  :1;
	} bits;
} pmbus_status_cml_t;

// Status Other
typedef union
{
	uint8_t all;
	struct
	{
		unsigned    st_dcok      	:1;
		unsigned    st_rfu1 		:1;
		unsigned    st_rfu2    		:1;
		unsigned    st_rfu3 		:1;

		unsigned    st_ext_wdt 		:1;
		unsigned    st_hw_imb 		:1;
		unsigned    st_cap_ovp 		:1;
		unsigned    st_rfu7  	    :1;
	} bits;
} pmbus_status_other_t;

// Status Mfr Specific
typedef union
{
	uint8_t all;
	struct
	{
		unsigned    st_hw_ocp      	:1;
		unsigned    st_hw_ovp 		:1;
		unsigned    st_sw_ocp    	:1;
		unsigned    st_sw_ovp 		:1;

		unsigned    st_sw_short1	:1;
		unsigned    st_sw_short2	:1;
		unsigned    st_sw_short3	:1;
		unsigned    st_ocpx  	    :1;
	} bits;
} pmbus_status_mfr_specific_t;


// Status Fan
typedef union
{
	uint8_t all;
	struct
	{
		unsigned    st_airflow_warn	 :1;
		unsigned    st_airflow_fault :1;
		unsigned    st_fan2_ovr      :1;
		unsigned    st_fan1_ovr      :1;

		unsigned    st_fan2_warn 	 :1;
		unsigned    st_fan1_warn 	 :1;
		unsigned    st_fan2_fault    :1;
		unsigned    st_fan1_fault    :1;
	} bits;
} pmbus_status_fan12_t;

typedef union
{
	uint8_t all;
	struct
	{
		unsigned    st_rfu0	 		:1;
		unsigned    st_rfu1 		:1;
		unsigned    st_fan4_ovr     :1;
		unsigned    st_fan3_ovr     :1;

		unsigned    st_fan4_warn 	:1;
		unsigned    st_fan3_warn 	:1;
		unsigned    st_fan4_fault   :1;
		unsigned    st_fan3_fault   :1;
	} bits;
} pmbus_status_fan34_t;

// ********************************
// PMBUS System register definition
// ********************************
// DSP System Reset Status
typedef struct
{
	uint8_t rcm_srs0;		// reset
	uint8_t rcm_srs1;
	uint8_t rcm_ssrs0;		// sticky
	uint8_t rcm_ssrs1;		// sticky
	uint8_t smc_pmstat;		// power mode status
	uint8_t pmc_lvdsc1;		// low voltage detect status
	uint8_t pmc_lvdsc2;
	uint8_t pmc_regsc;
	uint8_t wdog_rstcnt0;	// Watchdog Reset Count register LSB
	uint8_t wdog_rstcnt1;	// Watchdog Reset Count register MSB
}system_reset_status_t;

//store the valid values, ready for pmbus process
typedef struct
{
    uint8_t  firmware_revision;

    uint8_t  vout_mode;
	uint16_t vout;

    uint16_t iout;
    uint16_t iout1;
    uint16_t iout2;
    uint16_t iout3;
    uint16_t iprog;

    uint16_t pout;
    uint16_t pout_max;
    uint16_t vin;

    uint16_t temperature_pfc1;
    uint16_t temperature_pfc2;
    uint16_t temperature_pfc3;

    uint16_t temperature_sec1;
    uint16_t temperature_sec2;
    uint16_t temperature_sec3;

    uint16_t fan_ctrl;                           //control duty from thermal control logic in duty 0 to 100%
    uint16_t fan1_speed;
    uint16_t fan2_speed;
    uint16_t fan3_speed;
    uint16_t fan4_speed;

    uint16_t application_status;

    uint8_t on_off_config;
    uint8_t store_default_code;
    uint8_t restore_default_code;
    uint8_t store_user_code;

    uint8_t restore_user_code;
    uint8_t boot_loader;
    uint8_t operation;
    uint8_t write_protect0;

    uint8_t vout_ov_fault_response;
    uint8_t fan_config_1_2;
    uint8_t fan_config_3_4;

    pmbus_status_word_t		status_word;
    pmbus_status_vout_t     status_vout;
    pmbus_status_iout_t     status_iout;
    pmbus_status_input_t    status_input;
    pmbus_status_temperature_t status_temperature;
    pmbus_status_cml_t      status_cml;
    pmbus_status_other_t    status_other;
    pmbus_status_fan12_t    status_fan_1_2;
    pmbus_status_fan34_t    status_fan_3_4;
    pmbus_status_mfr_specific_t status_mfr_specific;

    uint16_t debug0;
    uint16_t debug1;
    uint16_t debug2;
    uint16_t debug3;

    uint16_t debugParam0;
    uint16_t debugParam1;

    uint16_t temperature_max_sec;
    uint16_t temperature_max_pri;
} pmbus_data_t;

typedef struct
{
	uint16_t power_up_ms;
	uint16_t vout_uvfd_ms;
	uint16_t vout_ovfd_ms;
	uint16_t vout_ovwd_ms;
	uint16_t vout_uvwd_ms;
	uint16_t iout_ocwd_ms;
	uint16_t iout_ocfd_ms;

	uint16_t vout_uv_max_delay_ms;
	uint16_t iout_oc_max_delay_ms;
	uint16_t vout_uv_max_restart_delay_10ms;
	uint16_t iout_oc_max_restart_delay_10ms;
	uint16_t ioutx_max_restart_delay_10ms;

	uint8_t  ot_fault_pfc1_delay;
	uint8_t  ot_fault_pfc2_delay;
	uint8_t  ot_fault_pfc3_delay;
	uint8_t  ot_warn_pfc1_delay;
	uint8_t  ot_warn_pfc2_delay;
	uint8_t  ot_warn_pfc3_delay;

	uint8_t  ot_fault_sec1_delay;
	uint8_t  ot_fault_sec2_delay;
	uint8_t  ot_fault_sec3_delay;
	uint8_t  ot_warn_sec1_delay;
	uint8_t  ot_warn_sec2_delay;
	uint8_t  ot_warn_sec3_delay;

	uint8_t  ot_fault_pri_delay;

	uint16_t timer_50ms;
	uint16_t soft_start_ocp_disable_delay_ms;

	uint16_t pmbus_timeout;
} pmbus_timer_t;

typedef struct
{
	uint16_t vout_command;

	uint16_t vout_uv_fault;
	uint16_t vout_uv_warn;
	uint16_t vout_ov_warn;
	uint16_t vout_ov_fault;			// all are in linear format when applicable

	uint16_t iout_oc_warn;
	uint16_t iout_oc_fault;
	uint16_t iout_oc_lv_fault;
	uint16_t iout_prog;

	uint16_t ot_sec_warn;
	uint16_t ot_sec_fault;
	uint16_t ot_pri_warn;
	uint16_t ot_pri_fault;

    uint16_t fan_command1;       	//duty 0~100

    uint16_t vin_uv_fault;
	uint16_t vin_uv_warn;
	uint16_t vin_ov_warn;
	uint16_t vin_ov_fault;

	uint8_t vout_uv_fault_response;
	uint8_t iout_oc_fault_response;
	uint8_t ot_fault_response;
	uint8_t vin_uv_fault_response;
	uint8_t vin_ov_fault_response;
	uint8_t ioutx_fault_response;

	uint8_t operation;
	uint8_t slaveID;			// slave ID, 0x00, means use default 0xB0 with A2-A0, if different than 0x00, use this address ID
	uint8_t slaveBaseAdr;		// Slave Base Address (0xB0 by default in conjunction with A2-A0), use only when slaveID = 0
	uint8_t rfu;

    uint16_t ramp_up_voltage;
    uint16_t ramp_down_voltage;
    uint16_t ramp_up_current;
    uint16_t ramp_down_current;

    uint16_t rfu2;
    uint16_t rfu3;
} pmbus_config_t;

// structure is use to set the serial port for modbus or SCPI (UART, RS232, RS485)
typedef struct
{
	uint32_t baudrate; 	// serial baudrate
	uint8_t stopbit;	// 0: 1 stop, 1: 2 stop bits
	uint8_t parity;		// 0:none, 2:even, 3:odd
	uint8_t databits;	// 0:8 bits, 1:9 bits
	uint8_t flags;		// flags
}comm_config_t;

// user configuration bits
typedef union
{
	uint16_t all;
	struct
	{
		unsigned cfg_current_walkin		:1;		// use only current soft start
		unsigned cfg_soc_disable		:1;		// disable safe operating soft start, go voltage/current walkin
		unsigned cfg_preload_disable	:1;		// pre-load is disable
		unsigned cfg_fan_off			:1;		// fan off during OFF state

		unsigned cfg_acok_logic			:1;		// 0:(0->good ; 1->bad)		1:(1->good ; 0-> bad)
		unsigned cfg_dcok_logic			:1;		// 0:(0->good ; 1->bad)		1:(1->good ; 0-> bad)
		unsigned cfg_slow_vprog			:1;		// 0: Standard VPROG filter 1: Slow VPROG filter, reduce noise in output due to VPROG signal
		unsigned cfg_fanok_logic		:1;		// 0:(0->good ; 1->bad)		1:(1->good ; 0-> bad)

		unsigned cfg_global_sync		:1;		// 0: AC Turn ON sync disable 			1: AC Turn ON Sync enable
		unsigned cfg_remote_inhibit_logic :1;	// 0:(signal=0->OFF ; signal=1->ON) 	1:(signal=0->ON ; signal=1->OFF)
		unsigned cfg_potentiometer_disable:1;	// 0:potentiometer output adj enable 	1:potentiometer output adj disable
		unsigned cfg_potentiometer_full_adj:1;	// 0:potentiometer adj +-10%			1: potentiometer adj 0% t0 105%

		unsigned cfg_analog_prog		:1;		// 0:pmbus prog							1:analog vprog/iprog
		unsigned cfg_disable_analog_iprog	:1;	// 0: use iprog signal in analog mode ; 1: use digital current limit when analog prog is enable, will only use vprog signal with digital current limit
		unsigned cfg_disable_analog_vprog	:1;	// 0: use vprog signal in analog mode ; 1: use digital voltage if analog prog is enable, will only use iprog signal with digital voltage setting
		unsigned cfg_no_preload_in_SD	:1;		// 0: preload is enable during SD 		1: preload not active during SD (0 -> preload will discharge capacitor at light load during SD sequence)
	} bits;
} user_config_t;

// hardware configuration bits
typedef union
{
	uint8_t all;
	struct
	{
		unsigned cfg_hw_scpi_protocol		:1;		// 0: use modbus protocol 1: use scpi protocol
		unsigned cfg_hw_rs485_half_duplex	:1;		// 0: RS485 full duplex   1: RS485 half duplex
		unsigned cfg_hw_rfu0				:1;		//
		unsigned cfg_hw_rfu1				:1;		//

		unsigned cfg_hw_rfu2				:1;		//
		unsigned cfg_hw_rfu3				:1;		//
		unsigned cfg_hw_rfu4				:1;		//
		unsigned cfg_hw_rfu5				:1;		//
	} bits;
} hardware_config_t;

typedef struct
{
	int16_t vout_uv_warn_cnt;			// vout adc
	int16_t vout_uv_fault_cnt;			// vout adc
	int16_t vout_ov_warn_cnt;			// vout adc
	int16_t vout_ov_fault_cnt;			// vout adc
	uint16_t vout_fault_ov_dac_ref;

	int16_t iout_oc_lv_fault_cnt;		// vout adc
	int16_t iout_oc_warn_cnt;			// iout adc
	int16_t iout_oc_fault_cnt;			// iout adc

	int16_t ot_sec_warn;				// T
	int16_t ot_sec_fault;				// T

	int16_t ot_pri_warn;				// T
	int16_t ot_pri_fault;				// T

	int16_t vin_uv_warn;
	int16_t vin_uv_fault;
	int16_t vin_ov_warn;
	int16_t vin_ov_fault;

	int16_t fan_command1;
	int16_t ramp_up_voltage;
	int16_t ramp_down_voltage;
    int16_t ramp_up_current;
    int16_t ramp_down_current;
} pmbus_real_mfr_config_t;

typedef union
{
	uint32_t all;
	struct
	{
		// MASK
		unsigned  ac1_bad	   	      		:1;
		unsigned  ac2_bad	   	      		:1;
		unsigned  ac3_bad	   	      		:1;
		unsigned  ac_imbalance	  			:1;
		unsigned  pfc1_bad	  				:1;
		unsigned  pfc2_bad	  				:1;
		unsigned  pfc3_bad	  				:1;
		unsigned  pri_over_voltage    		:1;

		unsigned  pwm_fault_ovp 	  		:1;	// set by PWMA fault, clear by ACF, restart, pmbus on/off (pwm fault 1)
		unsigned  pwm_fault_user_ovp		:1;	// set by dsp COMPARATOR# 2 (with vout), clear by ACF, restart, pmbus on/off (pwm fault 2)
		unsigned  sw_over_voltage 			:1; // set by pmbus ovp fault limit
		unsigned  hw_fault					:1;	// set when EXT_WDT or HW_IMB or CAP_OVP fault
		unsigned  ioutx_imbalance			:1;	// set when HV retries exhausted, clear by ACF, restart, pmbus on/off
		unsigned  pwm_fault_ocp				:1;	// short detection
		unsigned  sw_over_current      		:1;	// set by pmbus oc limit
		unsigned  fault_hold 				:1;	// hold fault until the restart timer expire

		unsigned  under_voltage       		:1; // set by fixed threshold, clear by ACF, restart, pmbus on/off
		unsigned  latched_fault	  			:1;	// latched due to input, OTP, FAN or PRICOMM
		unsigned  pfc1_otp 					:1;
		unsigned  pfc2_otp 					:1;
		unsigned  pfc3_otp 					:1;
		unsigned  sec1_otp 					:1;
		unsigned  sec2_otp 					:1;
		unsigned  sec3_otp 					:1;

		unsigned  fan1_fail			  		:1;
		unsigned  pfc_otp					:1;
		unsigned  remote_off  				:1;
		unsigned  pmbus_off					:1;
		unsigned  ioutx_ocp					:1; // not used yet
		unsigned  primary_comm1				:1;	// set by primary communication timeout
		unsigned  primary_comm2				:1;
		unsigned  primary_comm3				:1;
	} bits;
} shutdown_event_t;

#define SD_MASK_INPUT		0xE000007Fu
#define SD_MASK_INPUT_OV	0x00000080u
#define SD_MASK_PRI_COMM	0xE0000000u
#define SD_MASK_AC			0x00000007u
#define SD_MASK_AC_IMBALANCE 0x00000008u
#define SD_MASK_PFC			0x00000070u
#define SD_MASK_AC_PFC		0x000000FFu
#define SD_MASK_INP_WARNING	0xE00000FFu		// give AC_OK signal early warning before shutdown for these flags
#define SD_MASK_OVP			0x00000700u
#define SD_MASK_SHORT		0x00002000u		// short circuit
#define SD_MASK_OCP			0x00004000u		// current limit
#define SD_MASK_OCPX		0x10001000u
#define SD_MASK_UVP			0x00010000u
#define SD_MASK_RESET		0x00017F00u
#define SD_MASK_OTP			0x02FC0000u
#define SD_MASK_FAN			0x01000000u
#define SD_MASK_PFC_OTP		0x001C0000u
#define SD_MASK_SEC_OTP		0x00E00000u
#define SD_MASK_SW_OV		0x00000600u
#define SD_USER_OFF			0x0C000000u

typedef union
{
	uint32_t all;
    struct
    {
		unsigned in_current_limit				:1;
		unsigned in_power_limit					:1;
		unsigned ac_high_line					:1;
		unsigned pwm_enable						:1;

		unsigned ot_warning						:1;
		unsigned ot_fault						:1;
		unsigned temp_too_cold					:1;
		unsigned fan_fault						:1;

		unsigned rfu01							:1;
		unsigned under_voltage					:1;
		unsigned analog_prog					:1;
		unsigned vstby_enable					:1;

		unsigned current_walkin    				:1;
		unsigned hw_ovp_test_mode 				:1;
		unsigned sec_calibration_mode			:1;
		unsigned pri_calibration_mode			:1;

		unsigned DC_bus_adj						:1;
		unsigned rfu00							:1;
		unsigned preload_disable				:1;
		unsigned preload_on						:1;

		unsigned soc_soft_start					:1;
		unsigned ishare_enable					:1;
		unsigned scpi_enable					:1;
		unsigned dc_ok_fast						:1;

		unsigned sync							:1;
		unsigned pfc_otp						:1;
		unsigned pfc1_otp						:1;
		unsigned pfc2_otp						:1;

		unsigned pfc3_otp						:1;
		unsigned sec1_otp						:1;
		unsigned sec2_otp						:1;
		unsigned sec3_otp						:1;
    } bits;
} status_sys_t;

#define STATUS_MASK_OTP 0xFE000000u

typedef union
{
	uint32_t all;
    struct
    {
		unsigned fast_pri_comm					:1;
		unsigned sf_clear_fault_flag0		    :1;
		unsigned primary_pkt_received			:1;
		unsigned primary_fault_checked			:1;

		unsigned fan_hiccup_on					:1;
		unsigned ramp_down						:1;
		unsigned overshoot_jump					:1;
		unsigned ramp_up						:1;

		unsigned preload_ramp_down_ctrl_en		:1;
		unsigned force_current_loop				:1;
		unsigned voltage_walkin					:1;
		unsigned early_input_warning			:1;

		unsigned uart_setting_update			:1;
		unsigned canbus_setting_update			:1;
		unsigned vprog_fast_ramp				:1;
		unsigned ramp_up_from_low_vout			:1;
//----------
		unsigned undershoot_jump				:1;
		unsigned short3_jump					:1;
		unsigned pfc_manual_control				:1;
		unsigned input_sync_signal				:1;

		unsigned power_up_delay_done			:1;
		unsigned preload_extended				:1;
		unsigned preload_overload				:1;
		unsigned temp_cold						:1;

		unsigned rfu0							:1;
		unsigned control_isr					:1;
		unsigned test0                          :1;
		unsigned test1                          :1;
    } bits;
} system_flag_t;

extern pmbus_timer_t pmbus_timer;
extern pmbus_real_mfr_config_t pmbus_real_mfr_config;
extern pmbus_data_t pmbus_data;
extern shutdown_event_t shutdown_event;
extern shutdown_event_t last_shutdown_event;
extern shutdown_event_t shutdown_event_mask;
extern status_sys_t status;
extern system_flag_t system_flag;

extern system_reset_status_t system_reset_status;
extern uint8_t pmbus_address;

#define check_over_temperature(var, target, counter, max_counter, bitflag, recover, warnflag) { \
	if ((int32_t)var >= (int32_t)target)\
	{\
		if (counter > max_counter)\
		{\
			bitflag = 1;\
			pmbus_data.status_temperature.bits.st_temp_ot = 1;\
		}\
		else\
		{\
			if(counter > 10u)\
			{\
				warnflag = 1;\
			}\
			counter++;\
		}\
	}\
	else\
	{\
		warnflag = 0;\
		counter = 0;\
		if(var <= recover)\
		{\
			bitflag = 0u;\
		}\
	}\
}

#define check_warning_temperature(var, target, counter, max_counter, bitflag) { \
	if ((int32_t)var >= (int32_t)target) \
	{ \
		if (counter > max_counter) \
		{ \
			bitflag= 1; \
		} \
		else \
		{ \
			counter++; \
		} \
	} \
	else \
	{ \
		counter = 0; \
	} \
}

#define check_over_band(var, target, counter, bitflag, timeout,band) { \
	if (var > target) \
	{\
		if (counter == 0u) \
		{\
			bitflag= 1; \
			counter = timeout; \
		}\
	}\
	else if(var < (target - band)) \
	{\
		bitflag = 0; \
		counter = timeout; \
	} \
}

#define check_over_no_band(var, target, counter, bitflag, timeout) { \
	if (var > target) \
	{\
		if (counter == 0u) \
		{\
			bitflag= 1;\
			counter = timeout;\
		}\
	}\
	else \
	{\
		counter = timeout;\
	}\
}

#define check_under_band(var, target, counter, bitflag, timeout, band) { \
	if (var < target) \
	{\
		if (counter == 0u) \
		{\
			bitflag= 1;\
			counter = timeout;\
		}\
	}\
	else if(var > (target + band))\
	{\
		bitflag = 0;\
		counter = timeout;\
	}\
	else \
	{ \
		/* Hysteresis, do nothing*/ \
	} \
}

#define check_under_no_band(var, target, counter, bitflag, timeout) { \
	if (var < target) \
	{\
		if (counter == 0u) \
		{\
			bitflag= 1; \
			counter = timeout; \
		} \
	} \
	else \
	{ \
		counter = timeout; \
	} \
}

#define check_status(condition, bitflag) { \
	if (condition) \
	{\
		bitflag = 1;\
	}\
}

#define NO_INDEX 0u
#define VOUT_COMMAND_LIMIT_INDEX 1u
#define VOUT_OV_FAULT_LIMIT_INDEX 2u
#define VOUT_OV_WARN_LIMIT_INDEX 3u			// VOUT section using RVU and converted to cnt (calibrated)
#define VOUT_UV_FAULT_LIMIT_INDEX 4u
#define VOUT_UV_WARN_LIMIT_INDEX 5u
#define IOUT_OC_LV_FAULT_LIMIT_INDEX 6u

#define IOUT_OC_FAULT_LIMIT_INDEX 7u			// IOUT section using RVI and converted to cnt (calibrated)
#define IOUT_OC_WARN_LIMIT_INDEX 8u

#define FAN_COMMAND_LIMIT_INDEX 9u
#define OT_FAULT_LIMIT_INDEX 10u				// section are converted from linear pmbus to real value
#define OT_WARN_LIMIT_INDEX 11u
#define OT_PRI_FAULT_LIMIT_INDEX 12u			// section are converted from linear pmbus to real value
#define OT_PRI_WARN_LIMIT_INDEX 13u

#define VIN_OV_FAULT_LIMIT_INDEX 14u
#define VIN_OV_WARN_LIMIT_INDEX 15u
#define VIN_UV_FAULT_LIMIT_INDEX 16u
#define VIN_UV_WARN_LIMIT_INDEX 17u
#define RAMP_UP_VOLTAGE_LIMIT_INDEX 18u
#define RAMP_DOWN_VOLTAGE_LIMIT_INDEX 19u
#define RAMP_UP_CURRENT_LIMIT_INDEX 20u
#define RAMP_DOWN_CURRENT_LIMIT_INDEX 21u
#define MAX_LINEAR_LIMITS_INDEX 22u

#define VOUT_INDEX_MAX	IOUT_OC_LV_FAULT_LIMIT_INDEX
#define IOUT_INDEX_MAX	IOUT_OC_WARN_LIMIT_INDEX

typedef struct
{
    uint8_t type;
    uint8_t size;
    void *data_ptr;
    uint8_t data_limits_index;  	//index=0 means no limits
} command_t;

typedef struct
{
    uint8_t cmd_numbers;
    const command_t *cmd_table;
} cmd_class;

typedef union
{
    uint8_t byte[2];
    uint16_t word;
} byte2_word_t;

typedef struct
{
    int16_t *data_ptr;
    int32_t min;
    int32_t max;
    int16_t coeff;
    int32_t deflt;
} pmbus_limits_t;

extern const pmbus_limits_t pmbus_linear_limits[MAX_LINEAR_LIMITS_INDEX];
extern cmd_class pmbus_cmd[16];

#endif /* PMBUS_H_ */

