 /******************************************************************************
 *  File: pri_comm.h
 *  Description: primary communication function
 *  Author: Arnaud Hugron
 *  Date: 08/13/2015
 ****************************************************************************/

#ifndef PRI_COMM_H
#define	PRI_COMM_H

#ifdef	__cplusplus
extern "C" {
#endif

// command byte format
// bit7 bit6 bit5 bit4 bit3 bit2 bit1 bit0
// [A1] [A0] [ type  ] [     command     ]

// A1 and A0 is PFC unit ID. we have 3 pfc (0 -> 2), if A1, A0 is 1, cmd is for all (global write only)
// type: 00: read cmd; 01:write cmd

// primary PFC address ID
#define PRI_ADR_MASK_PFC0			0x00u
#define PRI_ADR_MASK_PFC1			0x40u
#define PRI_ADR_MASK_PFC2			0x80u
#define PRI_ADR_MASK_PFC_ALL		0xC0u

// primary communication commands (lower 4 bits only)
#define PRI_CMD_DATA				0x01u	// get pri_data_t structure from primary, read only
#define PRI_CMD_INFO				0x02u	// get pri_info_t structure from primary, read only
#define PRI_CMD_RUNTIME             0x03u   // get runtime
#define PRI_CMD_VIN_OFS             0x04u    // calibration ofs for Vin PK
#define PRI_CMD_DCBUS_OFS           0x05u    // calibration ofs for DCBUS
#define PRI_CMD_PFC_UV_LIMIT        0x06u    // set PFC UV early warning
#define PRI_CMD_SEC_STATUS			0x08u	// secondary status to primary (sec_status_t), global write only
#define PRI_CMD_DC_BUS_REF			0x09u	// send DC BUS reference to primary, write only
#define PRI_CMD_PFC_OFF             0x0Au    // turn OFF PFC, 2 byte write (0x55 0xAA)
#define PRI_CMD_PFC_ON              0x0Du    // turn ON PFC, 1 byte write (0x78)
#define PRI_CMD_CLEAR_FAULT			0x0Eu	// clear faults, 1 byte write (0xE0)
#define PRI_CMD_DC_BUS_GLOBAL_REF	0x0Fu	// send global DC BUS reference to primary, write only (send 3 bytes-> ref1, ref2, ref3), send all references in 1 cmd

// mask for command byte
#define PRI_SLAVE_ID_MASK          	0xC0u    // mask to indicate the PFC ID
#define PRI_WR_BIT_MASK        		0x10u    // bit that indicate a write command (0:read, 1:write)
#define PRI_OUTPUT_STATE_MASK		0x20u	// indicate to primary state of output
#define PRI_CMD_MASK               	0x0Fu    // cmd mask

#define PRI_WATCHDOG_100MS			80u		// in 100ms, we scan each PFC every 1s but watchdog is still counting during that 1s delay.

typedef union
{
	uint32_t all;
	struct
	{
		unsigned pfc_global_off 	:1;
		unsigned pfc_off1			:1;
		unsigned pfc_off2			:1;
		unsigned pfc_off3			:1;

		unsigned pfc_global_on		:1;
		unsigned pfc_on1			:1;
		unsigned pfc_on2			:1;
		unsigned pfc_on3			:1;

		unsigned pfc_vref1			:1;
		unsigned pfc_vref2			:1;
		unsigned pfc_vref3			:1;
		unsigned pfc_global_vref	:1;

		unsigned secondary_status 	:1;
		unsigned clear_fault		:1;
		unsigned rfu1				:1;
		unsigned rfu2				:1;

		unsigned pfc_uv_limit		:1;
	}bits;
}pri_comm_priority_cmd_t;

typedef union
{
	uint8_t all;
	struct
	{
		unsigned output_on 			:1; 	// Set/reset by secondary
		unsigned calibration_mode	:1;	 	// calibration/factory mode
		unsigned store_calibration	:1;		// save calibration
		unsigned pfc_all_ok			:1;		// all PFCs are ready
	}bits;
}secondary_status_t;

typedef struct
{
    uint16_t frame_error_cnt;
    uint16_t overrun_cnt;
    uint16_t pkt_crc_error_cnt;
    uint16_t pkt_timeout_cnt;
    uint8_t comm_watchdog_100ms;
    uint8_t rfu;
}sec_comm_err_t;

// TBD #pragma pack(push)
// TBD #pragma pack(1)							// pack below structures to 1

typedef union
{
    uint8_t all;
    struct
    {
        unsigned ac_uv              :1;	// input voltage under voltage
        unsigned ac_ov              :1;	// input voltage over voltage
        unsigned pfc_latch          :1; // pfc is latch due to hiccup retry or may be OVP
	    unsigned pfc_uv          	:1; // pfc DC bus under voltage

	    unsigned pfc_ov          	:1; // pfc DC bus is over voltage
	    unsigned pfc_too_low        :1;	// pfc DC bus is very low
	    unsigned over_temperature   :1; // over temperature
        unsigned pfc_remote_off     :1; // pfc is in off state
    }bits;
}pfc_fault_t;

typedef union
{
	uint8_t all;
    struct
    {
        unsigned pfc_ready          :1; // indicate to the secondary that the pfc is ready
	    unsigned relay_close        :1;	// relay is open
        unsigned ac_quick_fault     :1; // indicate a fast ac fault detection
        unsigned pfc_status         :1; // status of the PFC_ON pin

        unsigned high_line          :1; // indicate high line condition
        unsigned warn_temperature   :1; // temperature warning
        unsigned pfc_start_timeout  :1; // PFC was not able to reach expected voltage after the delay
        unsigned ac_loss   			:1; // AC loss indication
    }bits;
}pfc_status_t;

typedef struct
{
	uint16_t vin;
	uint16_t dc_bus;
	int16_t temperature_c;
	uint8_t dc_bus_ref;
	pfc_fault_t fault;
	pfc_status_t status;
}pfc_data_t;

typedef struct
{
    uint16_t frw_version;
    uint16_t pfc_state;
    uint16_t frame_error_cnt;
    uint16_t overrun_cnt;
    uint16_t pkt_err_cnt;
    uint16_t pkt_crc_error_cnt;
    uint16_t pkt_timeout_cnt;
    uint16_t relay_close_cnt;
    uint8_t last_pfc_fault;
    uint8_t rfu;
    uint16_t pfc_uv_limit;
}pfc_info_t;

typedef union
{
    uint32_t all;
    struct
    {
        uint16_t time0;
        uint16_t time1;
    }word;
}runtime_t;

// TBD #pragma pack(pop)							// restore pragma state

#define FAST_COMM_RESET_DELAY 70u

extern uint8_t pri_update_counter;
extern uint16_t reset_quick_acf;

extern pfc_info_t pfc_info1;
extern pfc_info_t pfc_info2;
extern pfc_info_t pfc_info3;
extern pfc_data_t pfc_data1;
extern pfc_data_t pfc_data2;
extern pfc_data_t pfc_data3;
extern sec_comm_err_t sec_comm_err1;
extern sec_comm_err_t sec_comm_err2;
extern sec_comm_err_t sec_comm_err3;
extern sec_comm_err_t *sec_comm_err;
extern secondary_status_t secondary_status;
extern pri_comm_priority_cmd_t pri_comm_priority_cmd;
extern uint8_t pfc_adj1;					// 50% duty cycle by default (0 -> 255)
extern uint8_t pfc_adj2;
extern uint8_t pfc_adj3;

extern runtime_t runtime;

void pri_comm_init_process(void);
void pri_comm_rx_process(void);
void pri_comm_tx_process(void);
void pri_comm_command_process(void);
void pri_comm_check_comm_timeout(void);

#ifdef	__cplusplus
}
#endif

#endif	/* PRI_COMM_H */
