/******************************************************************************
 *  File: timer.c
 *  Description: timer function
 *  Author: Arnaud Hugron
 *  Date: 08/13/2015
 ****************************************************************************/
#include "main.h"
#include "setup.h"
#include "timer.h"
#include "control.h"
#include "fan.h"
#include "pri_comm.h"
#include "temperature.h"
#include "digital_io.h"
#include "state.h"
#include "restart.h"
#include "flashInterface.h"
#include "primary.h"
#include "pmbus.h"
#include "power.h"
#include "preload.h"
#include "analog.h"
#include "calibration.h"


#define SECONDS_PER_HOUR		3600	// number of seconds in one hour

#ifdef OPEN_LOOP
uint8_t open_loop_state;
#endif

void timer_10ms_process(void);
static void timer_100ms_process(void);
static void timer_1s_process(void);

static inline void wdog_refresh(void)
{
    SysCtl_serviceWatchdog();       // Reset the watchdog counter, Watchdog is set to 104ms
}

void timer_init(void)
{
#ifdef OPEN_LOOP
	open_loop_state = 0;
#endif
}

// 250us periodic task
#pragma CODE_SECTION(timer_process, ".TI.ramfunc");
/*__attribute__ ((section(".mydata")))*/ void timer_process(void)
{
	static uint8_t loop_cnt = 0;

	if(IS_TIMER_SET())				// will be set every 250us
	{
		TIMER_CLEAR();              // clear timer flag


		if(loop_cnt++ >= 3u)
		{
			// every 1ms
			loop_cnt = 0;
			pmbus_timer_process();
			timer_10ms_process();	// run slower process
		}

#ifdef OPEN_LOOP
        open_loop_timer();
#endif
	}
}

// Called every 1ms
#pragma CODE_SECTION(timer_10ms_process, ".TI.ramfunc");
/*__attribute__ ((section(".mydata"))) */void timer_10ms_process(void)
{
	static uint16_t timer_10ms = 0;

	switch (timer_10ms)
	{
		case 0x00:
			timer_10ms = 9;							// reset timer
			timer_100ms_process();					// run 100 ms process
			if (restart.wait_timer_10ms > 0u)
			{
				restart.wait_timer_10ms--;
			}
			break;
		case 0x01:
			pri_comm_command_process();				// process communication command with primary
			timer_10ms--;
			break;
		case 0x02:
			// filter control word for PFC bus closed loop regulation
			control.ctrl_modulo_filtered = (uint16_t)((uint32_t)ctrl_modulo >> 16);
			timer_10ms--;
			break;
		case 0x03:
			// current limit programming (analog)
			if((status.bits.analog_prog == 1u) && (calibration_dynamic_ram.user_config.bits.cfg_disable_analog_iprog == 0u))
			{
				current.iset_ref = cal_iprog2ioutcnt(current.iprog_filt);	            // analog IPPROG (TBD from ST)
			}
			else
			{
				current.iset_ref = current.iset_ref_max;			// digital OCP
#ifdef OPEN_LOOP
				current.i_ref = current.iset_ref;
#endif
			}
			timer_10ms--;
			break;
		case 0x04:
			primary_process();
			timer_10ms--;
			break;
		case 0x05:
			check_iout();
			timer_10ms--;
			break;
		case 0x06:
			power_protection();
			flash_process();
			timer_10ms--;
			break;
		case 0x07:
			pmbus_data_process();
			timer_10ms--;
			break;
		case 0x08:
            wdog_refresh();                         // reset main watchdog
            vout_adjust_process();      // output adjustment using the potentiometer
			timer_10ms--;
			break;
		case 0x09:
			check_vout();
			timer_10ms--;
			break;
		default:
			timer_10ms--;
			break;
	}
}

// Called every 10ms
static void timer_100ms_process(void)
{
	static uint16_t timer_100ms = 0;

	switch (timer_100ms)
	{
		case 0x00:
			timer_100ms = 9;						// reset timer
			timer_1s_process();						// run 100 ms process
			break;
		case 0x01:
			timer_100ms--;
			break;
		case 0x02:
			pri_comm_check_comm_timeout();			// check communication timeout for PFC
			timer_100ms--;
			break;
		case 0x03:
			fan_pwm_process();
			power_internal_llc_ishare_process();
			timer_100ms--;
			break;
		case 0x04:
			current.iout_rvi_cal1 = cal_cnt2rvi(current.iout1_filt, 1);
	    	pmbus_data.iout1 = pmbus_to_linear_format(current.iout_rvi_cal1, (int8_t)IOUT_EXPONENT);
			timer_100ms--;
			break;
		case 0x05:
			current.iout_rvi_cal2 = cal_cnt2rvi(current.iout2_filt, 2);
	    	pmbus_data.iout2 = pmbus_to_linear_format(current.iout_rvi_cal2, (int8_t)IOUT_EXPONENT);
			timer_100ms--;
		    break;
		case 0x06:
			current.iout_rvi_cal3 = cal_cnt2rvi(current.iout3_filt, 3);
	    	pmbus_data.iout3 = pmbus_to_linear_format(current.iout_rvi_cal3, (int8_t)IOUT_EXPONENT);
            EXT_WATCHDOG_CLR()                      // End 10ms pulse for external WATCHDOG
			timer_100ms--;
		    break;
		case 0x07:
            EXT_WATCHDOG_SET()                      // start 10ms pulse for external WATCHDOG
			timer_100ms--;
		    break;
		default:
			timer_100ms--;
			break;
	}
}

// Called every 100ms
static void timer_1s_process(void)
{
	static uint16_t timer_1s = 0;
#ifndef OPEN_LOOP
	uint8_t max_ctrl_ocp_cnt;
#endif

	switch (timer_1s)
	{
		case 0x00:
			timer_1s = 9;							// reset timer
			break;
		case 0x01:
			check_system_reset_status();
			timer_1s--;
			break;
		case 0x02:
			if(restart.reset_timer_s > 0u)
			{
				restart.reset_timer_s--;
			}
			timer_1s--;
			break;
		case 0x05:
			timer_1s--;
			break;
		case 0x06:
			fan_process();				// calculate fan ctrl
			timer_1s--;
			break;
		case 0x07:
			temperature_process();
			timer_1s--;
			break;
		case 0x08:
			fan_speed();				// read tachometer from fan and get RPM value
			timer_1s--;
			break;
		default:
			timer_1s--;
			break;
	}
}

//***************************************************************************************************
#ifdef OPEN_LOOP
void open_loop_timer(void)
{
	switch (open_loop_state)
	{
		case 0:
			break;								// do nothing in state 0, controlled manually
		case 1:
			open_modulo += 0x100;
			if (open_modulo > FREQ_TO_COUNT(100000))
			{
				open_loop_state = 2;
			}
			break;
		case 2:
			open_modulo -= 0x100;
			if (open_modulo < MODULO_MIN_CTRL)
			{
				open_loop_state = 1;
			}
			break;
        case 3:
            open_loop_state = 4;
            open_modulo = pmbus_data.debugParam0;
            open_modulo <<= 16;
   //         DEBUG0_SIG_ON();
            break;
        case 4:
            open_loop_state = 3;
            open_modulo = pmbus_data.debugParam1;
            open_modulo <<= 16;
     //       DEBUG0_SIG_OFF();
            break;
        case 5:
        case 6:
            break;
        default:
			open_loop_state = 1;
			break;
	}
}
#endif 	// OPEN_LOOP
